
	
	; .macro StoreLE src,dest
		; mov r12,\src
		; strb r12,[\dest]
		; mov r12,r12,lsr #8
		; strb r12,[\dest,#1]
	; .endm 
	
	; .macro LoadLE src,dest
		; adrl r11,Mask16bit
		; ldr r11,[r11]
		
		; and \dest,\dest,r11
		; ldrb r12,[\src,#1]
		; mov r12,r12,lsl #8
		; ldrb r11,[\src]
		; add \dest,\dest,r11
		; add \dest,\dest,r12
	; .endm 
	
	; .macro loadLEA0 off,dest		;R1=A0
		; adrl r11,Mask16bit
		; ldr r11,[r11]
		; and \dest,\dest,r11
		
		; ldrb r12,[r10,#\off+1]
		; mov r12,r12,lsl #8
		; ldrb r11,[r10,#\off]
		; add \dest,\dest,r11
		; add \dest,\dest,r12
	; .endm

;R0	 A
;R1	 B BC 
;R2  D DE
;R3  H HL
;R4	 C
;R5	 E
;R5  L
;R7  IX  (Was A5)
;R8  IY  (Was A6) 
;R9 	 (Was D7)
;R10 	 (Was A0)

;ChibiVM simulates an 8 bit cpu, it's design is inspired mostly by the Z80 
;  and 6809	and will be therefore referred to as the AKUC809.R1 (chibi AKUmas Cpu 809 Revision 1)

;It uses the Z80's 8/16 bit register pair combination and core instruction set capabilities, flag functions
; including 16 bit stack pointer and IX+n type addressing - It is also LITTLE ENDIAN
;It uses the 6809's zeropage, three letter instructions mnemnonics, B accumulator and extended byte for flaxible addressing
;It uses the PDP-11's 8 register concept and powerful addressing modes (which also inspired the 6809)
;It uses 68000 style traps for interrupts
;It uses the 2 byte branch ops and vblank counter from Chip-8
;It uses the concept of memory mapped registers of the TMS9900
;It uses Memory mapping / bankswitching based on that of the Enterprise / Mastersystem

	
;Syntax

;First 256 bytes are a 'zeropage' - The zeropage is not relocatable
;First 16 of those bytes are 'register'
;0-7  are register pairs R0-R7
;8-16 are system (SP/PC Flags)
;17-31 are free for user use.
;32-47 are reserved for Virtual memory mapping (when enabled)

;Commands are defined by top 6 bits:
; %CCCCCCAA - C= 6 bit command A=2 bit addressing mode
; So 64 commands max!
; Commands %110000+ are reserved for user extensions.

; AA=0 Params are R0 and R2				 %CCCCCC00
; AA=1 Params are R0 and 8 bit immediate %CCCCCC01 %IIIIIIII
; AA=2 Params are R0 and Zero page Z  	 %CCCCCC10 %ZZZZZZZZ
; AA=3 uses extra Byteparam 			 %CCCCCC11 %PPPPDDDD  - P=Parameter2 D=Param1 & Dest
	;More bytes may be neded depending on parameter selection
	
;Branches use diferent addressing options:
;	AA=0 PC=PC+2
;	AA=1 PC=PC+IMM8
;	AA=2 PC=IMM16
;	AA=3 PC=R6/R7
	
;ByteParam supports 15 options for source & Destination.

;Reg 8 bit 0-3, 16 bit 4,6 (5,7 can be acessed via ZP)
;Zeropage Address / 16 bit address
;RegPair = 16 bit pointer
;RegPair+ = 16 bit pointer with PostInc
;RegPair with imm8 offset = 16 bit pointer with signed immediate offset

;For Parameter only: 	Immediate 8 bit / Immediate 16 bit 
;For Dest only: 		F register / Top of stack

;Warning! Some options may not make sense - Eg 8 bit load command with 16 bit immediate
;16 bit load with 8 bit immediate.
;Results will be unpredictable!

;Flags are in Zeropage:15 in the format ;%HSTEI-ZC
;Flag functions (and all functions in general) should mimic a z80
;Commands that do not set flags leave them unchanged.
;Commands That set one flag may not correctly set both flags, other flag is undefined

;Halt Flag: Set when the CPU halts, can be used to detect when CPU is not running (or check if current command is halt)
;			If you restart the CPU with a new task, you should clear the flags
;			If you HALT the system, you should assume it will never restart (your work is done)
;			If you want to be restarted, set S flag (Sleep) - to tell the host to restart at a good time (By stepping over the HALT command)

;Sleep Flag Use this to mark a HALT as possibly temporary if you want the host to restart your VM at some point

;Trap Flag: you can set this at the start of your traps, and clear it when done
;			This is intended so you can see if the CPU is running an interrupt - and give it higher priority 
;			If doing this, Note the return will not occur when the flag clears, so you should run one extra tick to process the RET command after the flag clears

;			Back up flags before trap with PHF
;			Set Flag with:    vm_ORR+ParamExtByt,Dest_ZeroPg+Param_Imm8,VM_rF,VM_fTrap  
;			Flags should restored with PLF 

;Error Flag 	Set when (1) an unknown command byte was detected
;				(2) A system call or trap was executed with an unsupported parameter (eg bankswitching on a machine that cannot do it)

;Interrupt Disable flag: (I)
;			Stop hardware from sending interrupt traps (Eg Vblank)



;Virtual Memory & Bankswitching:
;			The 64k address range is split into 4 blocks in the format
;			%BBAAAAAA AAAAAAAA BB=Bank AA...AA=Address 
;			4x 32 bit calues in the Zeropage define the Physical address base for each bank
;			This 32 bit value is added to the A bits to give the pysical address
; 			The correct 'normal' values are therefore &0000 &4000 &8000 &C000
;These are HARDWARE addresses, ChipEXT will provide a 'platform independant' bankswitching method 
;WARNING: 	Bankswitching occurs only once for each read/write.
;			This means reading a word spread across a 16k boundary may malfunction, as the second byte will be read from the first bank+1.
;			* Including words in code! *

;Zero page layout:

;First 8 bytes are registers:	AA KK BB CC DD EE HH LL (General Regs)
;Next 8 bytes are system:		FF FU PC PC SP SP VB VB (System Regs)
;Next 16 bytes are for user:    0  0  0  0  0  0  0  0  (User)
;...						    0  0  0  0  0  0  0  0	(User)
;Next 16 bytes are bankswitch   B0 B0 B0 B0 B1 B1 B1 B1 (Bankswitching)
;...						    B2 B2 B2 B2 B3 B3 B3 B3 (Bankswitching)

;Remaining bytes are implementation specific. Stack may share ZP ram. 
;Bankswitch bytes are only used if vm_useVMEM is defined 
; A recomended absolute minimum would be 64 bytes ram total for ZP+Stack

;Traps and systcalls use the 'parameter' to select a numbered function from the vector list
;These are intended to work from an 8 bit immediate, but can also work from a register or memory.

;value in register AK will be used to select 'subfunction' of trap - other regs are parameters as defined by that function

;Traps:
;0 RESET - Used to INIT the program counter
;1 VBLANK?
;3+ - For your own use

;Syscall: (suggestion)
;0 ChipEXT - reserved for system functions (Bankswitching / Debugger / Rand / Mult etc) - comamnd in A, Sets E flag if unknown command
;1 VMControl - For configuring VM functions (Enable/disable interrupts / Shutdown VM / Disk Load etc)
;2 VMMonitor - VM Monitor - dump regs to screen
;3+ - For your own use

    .equ VM_fHalt , 0b10000000	;HALT ocurred (Set Automatically)
    .equ VM_fSleep, 0b01000000	;Sleep (Set Manually)
    .equ VM_fTrap , 0b00100000	;Trap Running (Set Manually)
    .equ VM_fError, 0b00010000	;Unknown Opcode (Set Automatically) \ Syscall function \ Trap function
    .equ VM_fIntDis,0b00001000	;Interrupts disabled (Set Manually)
    .equ VM_fTrace, 0b00000100	;Trace enabled (monitor after each command)
    .equ VM_fZero,  0b00000010		;Last command resulted in Zero
    .equ VM_fCarry, 0b00000001		;Last command resulted in Carry

;All registers are 8 bit, all can be used as 16 bit pairs
;R0-R3 are intended for 8 bit ops (R2+3 is also ideal for 16 bit)
;R4+5 R6+7 are intended for 16 bit ops
;All are accessible from the Zero page addresses 0-7

;Registers Should be referred to by number, though they are given Z80 style 'Letter pairs' to make them easier to remember
;16 bit ops automatically use the next register (eg R0+R1)
;Acutally, odd combinations will also work in 16 bits (Eg R5+R6 LD), though the indirect addressing modes
;do not support these combos.


;When an operation is performed with no other parameter, it is performed on AB
; EG: XOR   performs A  = A  XOR B (R0 = R0 XOR R2)
;     ADD16 performs AK = AK + UB (R0/1 = R0/1 + R2/3)

    .equ VM_rR0,0	;Accumulator A - 16 bit AK pair  (Default Destination for ops)
    .equ VM_rR1,1	;K (Counter?)
    .equ VM_rR2,2	;Accumulator B  - 16 bit UB Pair  (Default parameter for ops)
    .equ VM_rR3,3	;U
    .equ VM_rR4,4	;E - 16 bit DE pair 
    .equ VM_rR5,5	;D
    .equ VM_rR6,6	;L - 16 bit HL Pair
    .equ VM_rR7,7	;H

    .equ R0,0
    .equ R1,1
    .equ R2,2
    .equ R3,3
    .equ R4,4
    .equ R5,5
    .equ R6,6
    .equ R7,7

    .equ VM_rF, 8	;Flags %HSTEItZC (Halted) (Halted -Sleeping) (Trap running) (Error - unknown opcode) (Interrupt disabled) (Zero) (Carry) (t=trace)
    .equ VM_rFU,9	;Flags User (unused by cpu - backed up with PHF / PLF) 
    .equ VM_rPC,10	;Program Counter
    .equ VM_rSP,12	;Stack pointer
    .equ VM_rVB,14	;Vblanks (0-65535)

    .equ vm_rBank0,32	;&0000-&3FFF 
    .equ vm_rBank1,36	;&4000-&7FFF
    .equ vm_rBank2,40	;&8000-&BFFF
    .equ vm_rBank3,44	;&C000-&FFFF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	.align 4
vm_ParameterAddressingModes:	;Subtract 5 (First addressing R0-R4)
	.long VM_GetAddress_R6							;vm_pR6,5	;Param R6
	.long VM_GetAddress_ByteParam_Indir16_R2_BaseImm	;vm_PB2,6 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - Base+imm8 offset
	.long VM_GetAddress_ByteParam_Indir16_R4_BaseImm	;vm_PB4,7 	;Indirect 16 bit pointer R4 (16 bit to anywhere) - Base+imm8 offset
	.long VM_GetAddress_ByteParam_Indir16_R2_PostInc	;vm_PU2,8 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - PostInc
	.long VM_GetAddress_ByteParam_Indir16_R6_PostInc	;vm_PU6,9 	;Indirect 16 bit pointer R6 (16 bit to anywhere) - PostInc
	.long VM_GetAddress_ByteParam_Indir16_R4			;vm_PQ4,10 	;Indirect 16 bit pointer R4 (16 bit to anywhere)
	.long VM_GetAddress_ByteParam_Indir16_R6			;vm_PQ6,11 	;Indirect 16 bit pointer R6 (16 bit to anywhere)
	.long VM_GetAddress_ByteParam_ImmAddr16			;vm_pA6,12	;Address 16
	.long VM_GetAddress_ByteParam_ImmAddr8			;vm_pA8,13	;Address 8 (ZeroPage)
	.long VM_GetAddress_ByteParam_Imm16				;vm_pI6,14	;Immediate 16 - Source only
	.long VM_GetAddress_ByteParam_Imm8				;vm_pI8,15	;Immediate 8 - Source only

	;.ifdef VASM
; vm_ParameterAddressingModesH,vm_ParameterAddressingModes>>8
; vm_ParameterAddressingModesL,vm_ParameterAddressingModes&255
	; else
; vm_ParameterAddressingModesH,vm_ParameterAddressingModes/256
; vm_ParameterAddressingModesL,vm_ParameterAddressingModes
	; endif	
	
;%-PPPFFF-
;Prep: (tasks run before command)
    .equ prp_Nothing,0*16
    .equ prp_VM_GetAddress_Branch ,1*16
    .equ prp_VM_GetAddress,2*16
    .equ prp_VM_GetAddress_VM_GetFlagC ,3*16
    .equ prp_VM_GetAddress16,4*16	
    .equ prp_VM_GetAddressPrep_ZPDest,5*16	;Patched for INC ZP (Dest=Param)
    .equ prp_VM_GetAddressPrep_ZPDest_GetFlagC,6*16
	
;Finishers: (tasks run after command)
    .equ fin_Nothing,0*2
    .equ fin_VM_PC_Store,1*2
    .equ fin_VM_PCINC,2*2
    .equ fin_VM_PCINC_FZ,3*2
    .equ fin_VM_PCINC_FZ_LD_IX_E,4*2
    .equ fin_VM_PCINC_FZ_LD_IX_DE,5*2
    .equ fin_VM_PCINC_LD_IX_E,6*2
    .equ fin_VM_PCINC_LD_IX_DE,7*2

	
vm_PrepVectors:				;Subtract 1 - Zero is nothing
	.long VM_GetAddress_Branch 				;1 - Branches
	.long VM_GetAddressPrep					;2 - Basic
	.long VM_GetAddress_VM_GetFlagC			;3 - Get Carry Flag
	.long VM_GetAddressPrep16				;4 - Patch for 16 bit imm
	.long VM_GetAddressPrepZPDest			;5 - Patch for INC ZP
	.long VM_GetAddressPrepZPDest_GetFlagC  ;6 patch for ROLz RORz
	
vm_FinVectors:				;Subtract 1 - Zero is nothing
	.long VM_PC_Store			;1 store PC
	.long VM_PCINC				;2 INC PC
	.long VM_PCINC_FZ			;3 store flags
	.long VM_PCINC_FZ_LD_IX_E	;4 store flags
	.long VM_PCINC_FZ_LD_IX_DE	;5 store flags and save 16 bit
	.long VM_PCINC_LD_IX_E 		;6 save
	.long VM_PCINC_LD_IX_DE 	;7 save 16 bit

	
vm_DecoderMatrix:
	.long cmd_NOP,		0
	.long cmd_PSH,		fin_VM_PCINC
	.long cmd_PUL,		fin_VM_PCINC
	.long cmd_PHF,		fin_VM_PCINC
	.long cmd_BRA,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_BSR,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_BEQ,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_BNE,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_BCS,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_BCC,		prp_VM_GetAddress_Branch+fin_VM_PC_Store
	.long cmd_ADD,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_SUB,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_CMP,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.long cmd_MOV,		prp_VM_GetAddress+fin_VM_PCINC_LD_IX_E
	.long cmd_STO,		prp_VM_GetAddress+fin_VM_PCINC
	.long cmd_LEA,		prp_VM_GetAddress+fin_VM_PCINC_LD_IX_DE
	.long cmd_NEG,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_ROL,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_ROR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_ASL,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_ASR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_LSR,		prp_VM_GetAddressPrep_ZPDest_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_INC,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_DEC,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_AND,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_ORR,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_XOR,		prp_VM_GetAddress+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_TRP,		prp_VM_GetAddress+fin_VM_PC_Store
	.long cmd_SYS,		prp_VM_GetAddress
	.long cmd_MOV16,	prp_VM_GetAddress16+fin_VM_PCINC_LD_IX_DE
	
	.long cmd_INC16,	prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	.long cmd_DEC16,	prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	
	.long cmd_ADD16,	prp_VM_GetAddress16+fin_VM_PCINC_FZ_LD_IX_DE
	.long cmd_SUB16,	prp_VM_GetAddress16+fin_VM_PCINC_FZ_LD_IX_DE
	.long cmd_SWP16,	prp_VM_GetAddress16+fin_VM_PCINC_LD_IX_DE
	.long cmd_STO16,	prp_VM_GetAddress16+fin_VM_PCINC
	.long cmd_BRA16,	prp_VM_GetAddress16+fin_VM_PC_Store	
	.long cmd_BSR16,	prp_VM_GetAddress16+fin_VM_PC_Store	
	.long cmd_TST,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.long cmd_CMP16,	prp_VM_GetAddress16+fin_VM_PCINC_FZ
	.long cmd_CLR,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_E
	.long cmd_CLR,		prp_VM_GetAddressPrep_ZPDest+fin_VM_PCINC_LD_IX_DE
	.long cmd_TOR,		prp_VM_GetAddress+fin_VM_PCINC_FZ
	.long cmd_ADC,		prp_VM_GetAddress_VM_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
	.long cmd_SBC,		prp_VM_GetAddress_VM_GetFlagC+fin_VM_PCINC_FZ_LD_IX_E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Instruction Set

    .equ vm_NOP,0b00000000	;No-Operation
    .equ vm_RET,0b00000001	;Return from sub
    .equ vm_HLT,0b00000010	;Stop Processor
    .equ vm_SEC,0b00000011	;Set Carry

    .equ vm_PH0,0b00000100
    .equ vm_PH2,0b00000101
    .equ vm_PH4,0b00000110
    .equ vm_PH6,0b00000111

    .equ vm_PL0,0b00001000
    .equ vm_PL2,0b00001001
    .equ vm_PL4,0b00001010
    .equ vm_PL6,0b00001011

    .equ vm_PHF,0b00001100
    .equ vm_PLF,0b00001101

    .equ vm_BRA,0b00010000
    .equ vm_BSR,0b00010100
    .equ vm_BEQ,0b00011000
    .equ vm_BNE,0b00011100
    .equ vm_BCS,0b00100000
    .equ vm_BCC,0b00100100
    .equ vm_ADD,0b00101000
    .equ vm_SUB,0b00101100
    .equ vm_CMP,0b00110000
    .equ vm_MOV,0b00110100			;Move Param to Dest
    .equ vm_STO,0b00111000			;Store Dest To Param
    .equ vm_LEA,0b00111100			;Load Effective address
    .equ vm_NEG,0b01000000
    .equ vm_ROL,0b01000100			;Rotate with carry
    .equ vm_ROR,0b01001000
    .equ vm_ASL,0b01001100
    .equ vm_ASR,0b01010000
    .equ vm_LSR,0b01010100
    .equ vm_INC,0b01011000
    .equ vm_DEC,0b01011100
    .equ vm_AND,0b01100000
    .equ vm_ORR,0b01100100
    .equ vm_XOR,0b01101000
    .equ vm_TRP,0b01101100			;Trap #PARAM
    .equ vm_SYS,0b01110000			;Syscall #PARAM
    .equ vm_MOV16,0b01110100
    .equ vm_INC16,0b01111000
    .equ vm_DEC16,0b01111100
    .equ vm_ADD16,0b10000000
    .equ vm_SUB16,0b10000100
    .equ vm_SWP16,0b10001000			;Swap 2 16 bit values
    .equ vm_STO16,0b10001100

    .equ vm_BRA16,0b10010000			;Branch To 16 bit Relative offset PARAM
    .equ vm_BSR16,0b10010100			;Branch To Sub 16 bit Relative offset PARAM
    .equ vm_TST  ,0b10011000
    .equ vm_CMP16,0b10011100

    .equ vm_CLR,  0b10100000
    .equ vm_CLR16,0b10100100
    .equ vm_TOR,  0b10101000			;Test OR
    .equ vm_ADC,  0b10101100	
    .equ vm_SBC,  0b10110000	
;Reserved for future use!
;vm_CMD,  %10110100			;Command switch
;vm_ADR,  %10111000			;Address switch
;vm_CPU,  %10111100			;Cpu switch
    .equ vm_LastComamnd,0b10110100

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Addressing Modes Branch:
 
    .equ vm_b2B,0 	;Branch PC+2 
    .equ vm_bI8,1	;Branch PC+Imm8 (Signed relative)
    .equ vm_bI6,2	;Branch PC+Imm16 (Absolute addr)
    .equ vm_bR6,3	;Branch R6/R7 (Absolute addr in R6/7)

;Addressing Modes Command:

    .equ vm_aAB,0	;AB Accumulator
    .equ vm_aIM,1	;Immediate
    .equ vm_aZP,2	;ZeroPage
    .equ vm_aBP,3	;ByteParam  %PARAM/DEST

;Addressing Modes Parameter:

;Syntax is z80 style eg:
;XOR A,B		A is destination B is parameter

    .equ vm_pR0,0	;Param R0
    .equ vm_pR1,1	;Param R1
    .equ vm_pR2,2	;Param R2
    .equ vm_pR3,3	;Param R3
    .equ vm_pR4,4	;Param R4
;vm_pR5,5	;Param R5 - Not available use zeropage
    .equ vm_pR6,5	;Param R6
;vm_pR7,7	;Param R7 - Not available - use zeropage
    .equ vm_PB2,6 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - Base+imm8 offset
    .equ vm_PB4,7 	;Indirect 16 bit pointer R4 (16 bit to anywhere) - Base+imm8 offset
    .equ vm_PU2,8 	;Indirect 16 bit pointer R2 (16 bit to anywhere) - PostInc
    .equ vm_PU6,9 	;Indirect 16 bit pointer R6 (16 bit to anywhere) - PostInc
    .equ vm_PQ4,10 	;Indirect 16 bit pointer R4 (16 bit to anywhere)
    .equ vm_PQ6,11 	;Indirect 16 bit pointer R6 (16 bit to anywhere)
    .equ vm_pA6,12	;Address 16
    .equ vm_pA8,13	;Address 8 (ZeroPage)
    .equ vm_pI6,14	;Immediate 16 - Source only
    .equ vm_pI8,15	;Immediate 8 - Source only

    .equ vm_pRF,15	;Flags Reg - Dest only
    .equ vm_pQS,14	;Indirect 16 bit pointer SP - Dest Only

;vm_PP2,8 	;Indirect 8 bit pointer R0 (8 bit to zeropage)
;vm_PP3,9 	;Indirect 8 bit pointer R2 (8 bit to zeropage)

    .equ vm_Ppr,16	;Parameter 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Friendly names

    .equ Param_R0,vm_pR0*vm_Ppr
    .equ Param_R1,vm_pR1*vm_Ppr
    .equ Param_R2,vm_pR2*vm_Ppr
    .equ Param_R3,vm_pR3*vm_Ppr
    .equ Param_R4,vm_pR4*vm_Ppr
    .equ Param_R6,vm_pR6*vm_Ppr
    .equ Param_Imm8,vm_pI8*vm_Ppr
    .equ Param_Imm16,vm_pI6*vm_Ppr
    .equ Param_Addr8,vm_pA8*vm_Ppr
    .equ Param_ZeroPg,vm_pA8*vm_Ppr
    .equ Param_Addr16,vm_pA6*vm_Ppr
    .equ Param_AtR4,vm_PQ4*vm_Ppr
    .equ Param_AtR6,vm_PQ6*vm_Ppr
    .equ Param_AtR2Inc,vm_PU2*vm_Ppr
    .equ Param_AtR6Inc,vm_PU6*vm_Ppr
    .equ Param_AtR2PlusImm8,vm_PB2*vm_Ppr
    .equ Param_AtR4PlusImm8,vm_PB4*vm_Ppr


    .equ Dest_R0,vm_pR0
    .equ Dest_R1,vm_pR1
    .equ Dest_R2,vm_pR2
    .equ Dest_R3,vm_pR3
    .equ Dest_R4,vm_pR4
    .equ Dest_R6,vm_pR6
    .equ Dest_Addr8,vm_pA8
    .equ Dest_ZeroPg,vm_pA8
    .equ Dest_Addr16,vm_pA6
    .equ Dest_AtR4,vm_PQ4
    .equ Dest_AtR6,vm_PQ6
    .equ Dest_AtR2Inc,vm_PU2
    .equ Dest_AtR6Inc,vm_PU6
    .equ Dest_AtR2PlusImm8,vm_PB2
    .equ Dest_AtR4PlusImm8,vm_PB4
    .equ Dest_RF,vm_pRF			;rF
    .equ Dest_AtSP,vm_pQS			;rStackTop



    .equ BranchPlus2,vm_b2B				;Skip 2 bytes (equivalent of BRAi 2)
    .equ BranchImm8,vm_bI8
    .equ BranchImm16,vm_bI6
    .equ BranchR6,vm_bR6

;Addressing Modes Command:

    .equ ParamAB,vm_aAB					;AK=Dest BC=Param
    .equ ParamImm8,vm_aIM				;Param= Immediate 8 bit (16 bit for 16 bit commands)
;ParamAddr8,vm_aZP ;ZeroPage
    .equ ParamZeroPg,vm_aZP ;ZeroPage	;Param in zero page (Dest for INC/DEC)
    .equ ParamExtByt,vm_aBP ;ByteParam  %PARAM/DEST - Advanced Addressing 16 options for SRC DEST


; End of VM code


;R12,R11 reserved as temporaries
;R10 is used as a base register.


    ;IX = R7 A5 
	;IY = R8 A6
		
VM_Tick:
	str lr,[sp,#-4]!		;put the Link register on the stack
		
	adrl r10,vm_rambaseAddr
	ldr r3,[r10]			;Get Base of VM Ram
	ldr r10,[r10]			;Get Base of VM Ram
	loadLEA0 vm_rpc,r3		;Load Little endian R3 from R10
							;(Program Counter)
	.ifdef vm_usevmem
		bl vm_virmemgethlbytea
	.else
		ldrb r0,[r3]		;Get A command from the Program
	.endif
	
	mov r2,r0				;D=Command
	
	and r0,r0,#0b11111100
	cmp r0,#vm_lastcomamnd
	bcs cmd_error			;Command not recognized
	

;Find the command in the decoder matrix	
	adrl r4,vm_decodermatrix
	add r4,r4,r0,lsl #1		;8 bytes per command
	
	
;calculate our finisher
	ldrb r0,[r4,#4]
	ands r0,r0,#0b00001110	;%-pppfff-
	beq vm_nofinish
		
	adrl r3,vm_finvectors-4
	ldr r0,[r3,r0,lsl #1]	;*4
	str r0,[sp,#-4]!		;Run this after the actual command
	
vm_nofinish:	


;Calculate our Comamnd Job
	ldr r0,[r4,#0]
	str r0,[sp,#-4]!		;Run the actual command
	
	
;Calculate our Prep Job
	ldrb r0,[r4,#4]
	ands r0,r0,#0b01110000
	beq vm_noprep
	
	adrl r3,vm_prepvectors-4
	ldr r0,[r3,r0,lsr #2];*4
	
	str r0,[sp,#-4]!	;run this before the actual command

vm_noprep:
	adrl r10,vm_rambaseAddr
	ldr r3,[r10]			;Get Base of VM Ram
	ldr r10,[r10]			;Get Base of VM Ram
	loadLEA0 vm_rpc,r3		;Load Little endian R3 from R10
							;(Program Counter)

	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr				;Execute it
	
	
cmd_error:
	mov r0,#vm_ferror	;set error flag.
	bl vm_setflaga
	b vm_pcinc			;skip unknown byte
	
	
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - push/pull has special rules
    ;IX = R7 A5 
	;IY = R8 A6

cmd_PHF:
	mov r0,#8
	bl cmd_Stack_GetRegA  	;R8=stack pointer  R7=points to reg R0
	
	tst r2,#0b00000001		;Addressing bit
	bne cmd_PULtoIX			;PULL
	b cmd_PSHfromIX 		;PUSH

cmd_PSH:
	bl cmd_Stack_GetReg	;R8=stack pointer  R7=points to reg R2
cmd_PSHfromIX:

	sub r8,r8,#2
	loadLE r7,r5			;Load R5 from R7 IX in LE order
	
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	
	storeLEa0 r8,vm_rsp		;Store Stack pointer

cmd_STO16:
VM_LD_IY_DE:	;Save 8 bit resuts, set flags and INC PC
VM_LD_IY_DEb:
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicaliy
			StoreLE r5,r3
		ldr r3,[sp],#4
		ldr lr,[sp],#4			;Pop return address (command)
		mov pc,lr 
	.else
		StoreLE r5,r8		;Store to R8 (IY)
		ldr lr,[sp],#4			;Pop return address (command)
		mov pc,lr 
	.endif
	
cmd_STO:
VM_LD_IY_E:	;Save 8 bit resuts, set flags and INC PC
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicaliy
			strb r5,[r3]
		ldr r3,[sp],#4
	.else
		strb r5,[r8] ;move.b d5,(a6)
	.endif
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
cmd_PUL:
	bl cmd_Stack_GetReg	;A6=stack pointer A5 points to reg D2
cmd_PULtoIX:
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicaliy
			ldrb r5,[r3],#1
			;addq.l #1,a3
			ldrb r2,[r3]
			
			bl vm_virmemgetphysicalix
			strb r5,[r3],#1
			;addq.l #1,a3
			strb r2,[r3]
			
		ldr r3,[sp],#4
	.else
		LoadLE r8,r5
		StoreLE r5,r7
	.endif
	
	add r8,r8,#2			;inc iy
	
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
		
	storeLEa0 r8,vm_rsp		;Store new SP
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	

cmd_Stack_GetReg:		;Get r2
	move r0,r2 
	and r0,r0,#0b00000011	;Bottom 2 (address bits) are reg pair
	mov r0,r0,lsl #1		;2 registers processed at a time

cmd_Stack_GetRegA:		;R8=stack pointer  R7=points to reg R0
	adrl r10,vm_rambaseAddr
	ldr r7,[r10]			;Get Base of VM Ram
	mov r8,r7
	mov r10,r7
	
	add.l r7,r7,r0			;R7 points to reg
	
	LoadLEA0 vm_rsp,r8		;R8 points to stack top
	mov pc,lr 
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - general commands

cmd_LEA:			
	mov r5,r8				;Return effective address of Parameter
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_MOV:
	mov r5,r4				;Parameter -> Destination
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_INC16:
	adrl r11,Mask16bitInv	;Set top 16 bits to 1
	ldr r11,[r11]
	orr r5,r5,r11			;Ensure carry is set if we go over 65535
	adds r5,r5,#1			;Add 1 and set flags
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_DEC16:
	adrl r11,Mask16bit		;Limit parameter to 16 bits
	ldr r11,[r11]
	and r5,r5,r11
	subs r5,r5,#1			;Subtract 1 and set flags
	bl FlipCarry			;Make carry work like Z80
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_ADD16:
	adrl r11,Mask16bitInv	;Set top 16 bits to 1
	ldr r11,[r11]
	orr r5,r5,r11			;Ensure carry is set if we go over 65535
	adds r5,r5,r4
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_SWP16:
	bl VM_LD_IY_DEb			;Store Dest value to Param address
	
cmd_MOV16:
	mov r5,r4				;Set Dest = Param
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_CLR:	
	mov r5,#0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
		
cmd_SBC:
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	adrl r11,Mask8bit		;Limit parameters to 8 bit
	ldr r11,[r11]
	and r5,r5,r11
	and r4,r4,r11
	sbcs r5,r5,r4			;Subtract with carry
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_SUB16:
	adrl r11,Mask16bit		;Limit parameters to 8 bit
	ldr r11,[r11]			
	and r5,r5,r11
	subs r5,r5,r4
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
		
cmd_ADD:
	adrl r11,Mask8bitInv
	ldr r11,[r11]
	orr r5,r5,r11			;Force carry on byte overflow
	adrl r11,Mask8bit
	ldr r11,[r11]
	and r4,r4,r11			;Limit parameter to 8 bit
	adds r5,r5,r4
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_ADC:
	adcs r5,r5,r4
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
cmd_CMP16:
	adrl r11,Mask16bit
	ldr r11,[r11]
	and r5,r5,r11			;Limit both parameters to 16 bit
	and r4,r4,r11
	cmp r5,r4
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
CarryR0:			;Set R0 to the carry flag
	mov r0,#0
	bcc CarryR0_0
	mov r0,#1
CarryR0_0:
	mov pc,lr

	
ZeroFlagFromByte:	;Set the zero flag from the byte in R5
	str lr,[sp,#-4]!
		bl MRSfake ;mrs r0,CPSR
		tst r5,#255		;Byte=0?	
		bne ZeroFlagFromByte1
		orr r0,r0,#0b01000000000000000000000000000000	;Yes? set Zero
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)	
	mov pc,lr	
		
ZeroFlagFromByte1:
		and r0,r0,#0b10110000000000000000000000000000	;Clear Zero
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	

	
CarryToBit8:		;Set bit 8 according to carry
	bcc CarryToBit8_0
	orr r5,r5,#0b100000000
CarryToBit8_0:
	mov pc,lr	
	
	
CarryFromBit8:		;Get the carry from bit 8
	str lr,[sp,#-4]!
		bl MRSfake ;mrs r0,CPSR
		tst r5,#0b100000000	;Bit pushed out of byte?
		beq CarryBit8_0
		orr r0,r0,#0b00100000000000000000000000000000	;Yes? set carry
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
CarryBit8_0:
		and r0,r0,#0b11010000000000000000000000000000	;clear carry
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
	
SetCarry:
	str lr,[sp,#-4]!
		bl MRSfake ;mrs r0,CPSR
		orr r0,r0,#0b00100000000000000000000000000000	
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
ClearCarry:
	str lr,[sp,#-4]!
		bl MRSfake ;mrs r0,CPSR
		and r0,r0,#0b11010000000000000000000000000000
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
FlipCarry:	;Carry works like 6502 (borrow with subtract)
			; , we need it to work like z80
	str lr,[sp,#-4]!
		bl MRSfake ;mrs r0,CPSR
		eor r0,r0,#0b00100000000000000000000000000000
		bl MSRfake ;msr CPSR,r0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

	
SignExtend8:	;Sign extend byte in R5 based on bit 7
	tst r5,#0b10000000
	beq SignExtend8_0
	
	adrl r11,Mask8bitInv
	ldr r11,[r11]
	orr r5,r5,r11		;Set bits 8+ to 1
	
SignExtend8_0:
	mov pc,lr
	
;this is the equvalent of MRS,
; we use it for OLD arm compatability (RiscOS)
MRSfake:
	mov r0,#0
	bcc MRSfakecc
	orr r0,r0,#0b00100000000000000000000000000000	;2=Carry 4=Zero 6=Z+C
MRSfakecc:	
	bne MRSfakez
	orr r0,r0,#0b01000000000000000000000000000000
MRSfakez:	
	mov pc,lr 

	
;this is the equvalent of MSR,
MSRfake:
	tst r0,#0b00100000000000000000000000000000 ;C
	bne MSRfake_c
	tst r0,#0b01000000000000000000000000000000 ;z
	bne MSRfake_z
	
MSRfake_none:	
	mov r10,#1			;Set nothing
	adds r10,r10,#1
	mov pc,lr 
	
MSRfake_c:	
	tst r0,#0b01000000000000000000000000000000 ;z
	bne MSRfake_cz
	mov r10,#3			;Set Carry
	subs r10,r10,#1
	mov pc,lr 
	
MSRfake_cz:	
	mov r10,#-1			;Set Carry+Zero
	adds r10,r10,#1
	mov pc,lr 
	
MSRfake_z:	
	mov r10,#0			;Set zero
	movs r10,r10,lsr #0
	mov pc,lr 


	
	
cmd_CMP: ;8 bit subtract / CMP
cmd_SUB:	
	adrl r11,Mask8bit
	ldr r11,[r11]
	and r5,r5,r11
	and r4,r4,r11
	subs r5,r5,r4
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_NEG:;
	rsb r5,r5,#0
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_ROL: ;(F:CZ)
	bl CarryR0				;Get flag in R0
	mov r5,r5,lsl #1		;Shift Dest Left
	add r5,r5,r0			;Add carry
	bl CarryFromBit8		;Set carry from bit 8
	bl ZeroFlagFromByte		;Set Zero flag from byte
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_ROR: ;(F:CZ)
	bl CarryToBit8			;Carry -> bit 8 of dest
	movs r5,r5,rrx			;Rotate Dest right
	bl ZeroFlagFromByte		;Set zero flag
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_ASL: ;(F:CZ)
	mov r5,r5,lsl #1		;Shift left 1 bit
	bl CarryFromBit8		;Set carry from bit 8
	bl ZeroFlagFromByte		;Set zero flag
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_ASR: ;(F:CZ)
	bl SignExtend8			;Fill bits 8+ with bit 7
	movs r5,r5,asr #1		;Arithematic shift right 1 bit
	bl ZeroFlagFromByte		;Set zero flag
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_LSR: ;(F:CZ)
	and r5,r5,#255			;Limit to 1 byte parameter
	movs r5,r5,lsr #1		;Logical shift righ 1 bit
	bl ZeroFlagFromByte		;Set zero flag
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_INC: ;(F:Z)
	adrl r11,Mask8bitInv
	ldr r11,[r11]
	orr r5,r5,r11			;Set bits 8+ to 1 (so carry works in 8 bit)
	adds r5,r5,#1			;add 1
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_DEC: ;(F:Z)
	adrl r11,Mask8bit
	ldr r11,[r11]
	and r5,r5,r11			;Limit parameter to 8 bit
	subs r5,r5,#1			;Subtract 1
	bl FlipCarry	;convert carry (ChibiVM/z80) to borrow (ARM/6502)
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_TST:
cmd_AND: ;(F:Z)
	and r5,r5,#255			;Limit parameter and dest
	and r4,r4,#255			; to 8 bits
	ands r5,r5,r4			;AND and set flags
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
cmd_TOR:
cmd_ORR: ;(F:Z)
	and r5,r5,#255			;Limit parameter and dest
	and r4,r4,#255			; to 8 bits
	orrs r5,r5,r4			;OR and set flags
	bl ClearCarry			;OR should always clear the carry
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

cmd_XOR: ;(F:Z)
	and r5,r5,#255			;Limit parameter and dest
	and r4,r4,#255			; to 8 bits
	eors r5,r5,r4			;EOR/XOR dest with param
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
; Finishing Routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	

	
VM_PCINC_LD_IX_DE:	;Save 16 bit results,  and INC PC
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicalix
			StoreLE r5,r3
		ldr r3,[sp],#4
		b vm_pcinc
	.else
		StoreLE r5,r7	;Store 16 bit result
		b vm_pcinc
	.endif
	
	
VM_PCINC_LD_IX_E:	;Save 8 bit results, and INC PC
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicalix
			strb r5,[r3]
		ldr r3,[sp],#4
	.else
		strb r5,[r7]	;Store 8 bit result
	.endif
	b vm_pcinc

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	


VM_PCINC_FZ_LD_IX_DE:	;Save 16 bit results, set flags and INC PC
	.ifdef vm_usevmem
		STMFD sp!,{r0,r7,r3}
			bl vm_virmemgetphysicalix
			StoreLE r5,r3
		LDMFD sp!,{r0,r7,r3}
		b VM_PCINC_FZ
	.else
		StoreLE r5,r7
		b VM_PCINC_FZ
	.endif
	
	
VM_PCINC_FZ_LD_IX_E:	;Save 8 bit results, set flags and INC PC
	.ifdef vm_usevmem
		STMFD sp!,{r0,r7,r3}
			bl vm_virmemgetphysicalix
			strb r5,[r3]
		LDMFD sp!,{r0,r7,r3}
	.else
		strb r5,[r7]
	.endif
	
	
VM_PCINC_FZ:			;Set flags and INC PC
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]					;Get Base of VM Ram
	ldrb r11,[r10,#vm_rf]
	and r11,r11,#0b11111110			;Clear VM Carry
	bcc vm_pcinc_ff2
		orr r11,r11,#0b00000001		;Set VM Carry	
VM_PCINC_FF2:

	and r11,r11,#0b11111101			;Clear VM Zero
	bne vm_pcinc_ff3
		orr r11,r11,#0b00000010		;Set VM Zero
VM_PCINC_FF3:
	strb r11,[r10,#vm_rf]
	
	
VM_PCINC:
	add r3,r3,#1			;PC++
		
VM_PC_Store:
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	storeLEa0 r3,vm_rpc		;Store PC
	
vm_pc_nostore:
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - these commands share the same 6 bit code 
;	addressing bits define which is the command

cmd_NOP:
	mov r0,r2
	ands r0,r0,#255
	beq vm_pcinc		;NOP	   0

	subs r0,r0,#1 		;cp vm_RET 1
	beq cmd_ret
						;cp vm_HLT 2
	subs r0,r0,#1
	bne cmd_HLT
	
	bl SetCarry			;SEC 3 
	
cmd_HLT:
	mov r0,#vm_fhalt		;Set Halt flag.
	bl vm_setflaga
	b vm_pc_store

cmd_RET:					;Also called RTS
	mov r0,#vm_rpc
	bl cmd_stack_getrega
	b cmd_pultoix			;get back pc from stack
	

VM_SetFlagA:		;set bits in vm f to the cpu a to 
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	ldrb r11,[r10,#vm_rf]
	orr r11,r11,r0
	strb r11,[r10,#vm_rf]
	mov pc,lr
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;command processors - branch commands have special addressing

cmd_BEQ:
	tst r0,#0b0000010		;Test virtual Zero flag
	bne cmd_bra_true
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts

cmd_BNE:
	tst r0,#0b0000010		;Test virtual Zero flag
	beq cmd_bra_true
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts

cmd_BCS:
	tst r0,#0b0000001
	bne cmd_bra_true
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts

cmd_BCC:
	tst r0,#0b0000001
	beq cmd_bra_true
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
cmd_BRA16:
	add r3,r3,#1
	add r3,r3,r4
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
cmd_bra_true:
cmd_BRA:
	mov r3,r7				;Perform Branch
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
	
vm_GetVector:				;Get 16 bit address R4 from table R7
	and r4,r4,#0xFF
	mov r4,r4,lsl #2		;4 bytes per address 
		
	ldr r4,[r7,r4]
	mov pc,lr 
	
	
VM_CauseTrapFromOutsideVM:		;c=trap number
	adrl r10,vm_rambaseAddr
	ldr r3,[r10]			;Get Base of VM Ram
	ldr r10,[r10]			;Get Base of VM Ram
	loadLEA0 vm_rpc,r3		;Load Little endian A3 from A0
							;(Program Counter)
							
	sub r3,r3,#1			;Nuter next add command!
				

cmd_TRP:					;D4=Trap Number
	add r3,r3,#1
	adrl r10,vm_trap_TableAddr
	ldr r7,[r10]
	and r4,r4,#0xFF
	mov r4,r4,lsl #1	;2 bytes per trap address (Address in vm)
	add r7,r7,r4
	
	adrl r10,vm_rambaseAddr
	ldr r4,[r10]			;Get Base of VM Ram
	
	ldrb r10,[r7,#1]		;Get the little endian address
	mov r10,r10,lsl #8		; from the trap table
	ldrb r0,[r7,#0]
	add r0,r0,r10
	add r4,r4,r0
	
	str r4,[sp,#-4]!		;Put the address to call on the stack
	b cmd_call
	
	
cmd_SYS:				;D4=Syscall Number
	bl vm_pcincBL
	
	adrl r7,vm_syscalls
		
	bl vm_getvector			;Vector r4, 4 bytes per address
	mov pc,r4 				;execute it
		
VM_PC_StoreBL:	
	str lr,[sp,#-4]!
	b VM_PC_Store
	
vm_pcincBL:
	str lr,[sp,#-4]!
	b vm_pcinc

cmd_PSHfromIXBL:
	str lr,[sp,#-4]!
	b cmd_PSHfromIX

	
cmd_BSR16:
	add r3,r3,#1
	mov r7,r3
	add r7,r7,r4		;Add 16 bit offset to PC
	

cmd_BSR:
	str r7,[sp,#-4]!

cmd_Call:
		bl VM_PC_StoreBL		;Store updated PC in VM
		mov r0,#vm_rpc
		bl cmd_stack_getrega	;Prepare to push vPC onto stack
		bl cmd_PSHfromIXBL		;Store the vPC on the VM stack
	ldr r3,[sp],#4
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Preparation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

VM_GetAddressPrepZPDest_GetFlagC:
	bl VM_GetAddressPrepZPDestBL	;Get a zeropage destination
	
	b VM_GetFlagC			;Get the flag C
	
VM_GetAddress_VM_GetFlagC:
	bl vm_getaddressprepBL	;Get the address
	
VM_GetFlagC:				;Set the CPU C flag to the VM one
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	ldrb r0,[r10,#vm_rf]
	movs r0,r0,rrx 			;Shift LSB into carry flag
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
VM_GetAddressPrep16:
	str r2,[sp,#-4]!
		bl vm_getaddressprepBL
	ldr r0,[sp],#4	
	
	and r0,r0,#0b00000011
	subs r0,r0,#1			;cp %00000001 (imm8 mode)
	bne lbl43438
	add r3,r3,#1			;extra byte for imm16

lbl43438:
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr
	

VM_GetAddressPrepZPDestBL:	
	str lr,[sp,#-4]!
VM_GetAddressPrepZPDest:	;Patched for Zeropage dest (INC ZP / INC16)	
	str r2,[sp,#-4]!
		bl vm_getaddressprepBL
	ldr r0,[sp],#4	
	
	and r0,r0,#0b00000011
	cmp r0,#0b00000010		;Zero page addressing?
	bne lbl31757
	mov r7,r8				;Move Param->Dest (addr)
	mov r5,r4				;Move Param->Dest (val)
lbl31757:
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
VM_GetAddressPrepBL:	
	str lr,[sp,#-4]!
VM_GetAddressPrep:
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	mov r7,r10				;Dest		R0
	mov r8,r10				;Param2		R2
	add r8,r8,#2

	ands r0,r2,#0b00000011		;0=Accumulator Addressing
	beq lbl39573	
		bl vm_getaddress	
lbl39573:

	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysicaliy
			LoadLE r3,r4	;D4=param (A6) / D5=dest (A5) / D2=command byte
			bl vm_virmemgetphysicalix
			LoadLE r3,r5 	;D4=param (A6) / D5=dest (A5)
		ldr r3,[sp],#4
	.else
		LoadLE r8,r4		;r4=param (r8) / r5=dest (r7) / r2=command byte
		LoadLE r7,r5 		;r4=param (r8) / r5=dest (r7) / r2=command byte
	.endif
	and r0,r2,#0b11111100	;Command bits
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Addressing mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

VM_GetAddress_ByteAddr:	;Zeropage source
	.ifdef vm_usevmem
		bl vm_virmemgethlbytea
	.else
		ldrb r0,[r3]		;Get a byte from the program
	.endif
	
	adrl r10,Mask8bitInv
	ldr r10,[r10]
	
	and r8,r8,r10			;Keep 0xFFFFF00 part
	add r8,r8,r0			;Replace bottom byte (Zero page address)

	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts

	
VM_GetAddress_IMM:		;8 bit immediate source (after HL / PC)
	mov r8,r3				;R3=virtual PC
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr ;rts
	
	
VM_GetAddress:			;R7Dest R8=Param
	str lr,[sp,#-4]!

	add r3,r3,#1
	subs r0,r0,#1		;cp  %00000001
	beq vm_getaddress_imm 
	subs r0,r0,#1		;cp  %00000010
	beq vm_getaddress_byteaddr
	
	
VM_GetAddress_ByteParam:	;Extended addressing modes
	.ifdef vm_usevmem
		bl vm_virmemgethlbytea	
	.else
		ldrb r0,[r3]		;Get A command from the Program
	.endif
	mov r5,r0
	mov r4,r7		;ppppDDDD
	bl VM_GetAddress_ByteParam2_Dest
	mov r7,r4
	mov r0,r5
	
	mov r0,r0,lsr #4
	mov r4,r8		;PPPP----
	
	bl VM_GetAddress_ByteParam2
	mov r8,r4		;----PPPP
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
		
;imm8 imm16 dest don't make sense for destination, here are some alternatives

VM_GetAddress_ByteParam2_Dest:
	str lr,[sp,#-4]!
	
	adrl r10,Mask8bitInv	;0xFFFFFF00
	ldr r10,[r10]
	
	and r0,r0,#0b00001111
	cmp r0,#vm_prf			;Flag Destination?
	bne notvm_getaddress_rf
		and r4,r4,r10
		add r4,r4,#vm_rf		;Dest=Flags register
		
		ldr lr,[sp],#4			;Pop return address (command)
		mov pc,lr 
		
notvm_getaddress_rf:
	cmp r0,#vm_pqs				;Stack Top Addressing?
	
	bne notVM_GetAddress_ByteParam_Indir16_RSP
		adrl r10,vm_rambaseAddr
		ldr r10,[r10]			;Get Base of VM Ram
		
		LoadLEA0 vm_rsp,r4		;Get Stack Address

		ldr lr,[sp],#4			;Pop return address (command)
		mov pc,lr 
		
notVM_GetAddress_ByteParam_Indir16_RSP:	
		b VM_GetAddress_ByteParam2b
	
	
VM_GetAddress_ByteParam2:
	str lr,[sp,#-4]!
	
VM_GetAddress_ByteParam2b:
	and r0,r0,#0b00001111

	adrl r10,Mask8bitInv	;0xFFFFFF00
	ldr r10,[r10]
	and r4,r4,r10			;Keep all but bottom byte 
	add r4,r4,r0			;  (Will be replaced)
	
	cmp r0,#vm_pr6
	bcc lbl65454			;0-4=direct register
	sub r0,r0,#5
	
	adrl r10,vm_ParameterAddressingModes
	ldr r0,[r10,r0,lsl #2]	;4 bytes per addressing mode
	mov pc,r0				;call the addressing mode

lbl65454:	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

	
	
	
VM_GetAddress_R6:
	adrl r10,Mask8bitInv	;0xFFFFFF00
	ldr r10,[r10]
	and r4,r4,r10
	add r4,r4,#6			;Reg R6/r7 is source
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	

vm_getaddress_byteparam_imm8:
	add r3,r3,#1
	mov r4,r3
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
VM_GetAddress_ByteParam_Imm16:
	add r3,r3,#1			;Move vPC to next byte
	mov r4,r3				;This byte is parameter address
	add r3,r3,#1

	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 


	
vm_getaddress_byteparam_immaddr8: ;Zero page addressing
	add r3,r3,#1
	.ifdef vm_usevmem
		bl vm_virmemgethlbytea
		adrl r10,Mask8bitInv	;0xFFFFFF00
		ldr r10,[r10]
		and r4,r4,r10			;Keep top 24 bits
		add r4,r4,r0
	.else
		ldrb r9,[r3]			;get 8 bit address
		adrl r10,Mask8bitInv	;0xFFFFFF00
		ldr r10,[r10]
		and r4,r4,r10			;Keep top 24 bits
		add r4,r4,r9
	.endif
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 


VM_GetAddress_ByteParam_ImmAddr16:
	add r3,r3,#1			;Skip first byte of code
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysical
			LoadLE r3,r4
		ldr r3,[sp],#4
	.else
		LoadLE r3,r4		;Get 16 bit address 
	.endif
	add r3,r3,#1			;Skip second byte of code
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr
	
	
	
VM_GetAddress_ByteParam_Indir16_R4: ;Indirect register pair addressing 
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
		
	LoadLEA0 vm_rr4,r4		;R4/R5
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr
	

VM_GetAddress_ByteParam_Indir16_R6:
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	
	LoadLEA0 vm_rr6,r4		;R6/R7
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
	
VM_GetAddress_ByteParam_Indir16_R2_PostInc: ;@Rn+ postinc
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	
	LoadLEA0 vm_rr2,r4		;Get vR2
	mov r0,r4
	add r0,r0,#1			;increment it
	StoreLEA0 r0,vm_rr2		;Store it back
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	

VM_GetAddress_ByteParam_Indir16_R6_PostInc:
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	
	LoadLEA0 vm_rr6,r4		;Get vR2
	mov r0,r4
	add r0,r0,#1			;increment it
	StoreLEA0 r0,vm_rr6		;Store it back
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
		
		

VM_GetAddress_ByteParam_Indir16_R2_BaseImm: ;(IX+n) type addressing
	add r3,r3,#1
	bl getsignextendedimm8		;D4=signed value from byte
	add r3,r3,#-1
	
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	mov r0,r10
	
	LoadLEA0 vm_rr2,r0		;Get R2
	add r4,r4,r0			;Add offset
	
	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 
	
	
VM_GetAddress_ByteParam_Indir16_R4_BaseImm:
	add r3,r3,#1
	bl getsignextendedimm8		;D4=signed value from byte
	add r3,r3,#-1
	
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]			;Get Base of VM Ram
	mov r0,r10
	
	LoadLEA0 vm_rr4,r0		;Get R4	
	add r4,r4,r0			;Add the offset

	ldr lr,[sp],#4			;Pop return address (command)
	mov pc,lr 

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Branch addressing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
VM_GetAddress_Branch:
	add r3,r3,#1
	mov r0,r2
	ands r0,r0,#0b00000011		;Bottom 2 bits denote branch type
	beq vm_getaddress_branch2	;Branch PC+2
	subs r0,r0,#1
	beq vm_getaddress_branch8imm	;1 branch pc+SignedImm8 (rel addr)
	subs r0,r0,#1
	beq vm_getaddress_branch16imm	;2 branch pc+imm16 (abs addr)
	
;VM_GetAddress_BranchR6:
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]				;Get Base of VM Ram

	loadLEA0 vm_rr6,r7			;Get R6->A5 (Branch Dest)
	b vm_getaddress_branchgetf
	
	
VM_GetAddress_Branch16imm:
	.ifdef vm_usevmem
		str r3,[sp,#-4]!
			bl vm_virmemgetphysical
			loadLE r3,r7
		ldr r3,[sp],#4
		add r3,r3,#2
	.else
		loadLE r3,r7			;Get 2 bytes from program
		add r3,r3,#2			;PC+=2
	.endif
	b vm_getaddress_branchgetf
	
	
VM_GetAddress_Branch8imm:
	bl getsignextendedimm8	 	;R4= -128 to +127
	mov r7,r3
	add r7,r7,r4				;Add offset to PC (Branch dest)

vm_getaddress_branchgetf:		;Jump to IX (R7)
	adrl r10,vm_rambaseAddr
	ldr r10,[r10]				;Get Base of VM Ram
	ldrb r0,[r10,#vm_rf]
	
	ldr lr,[sp],#4				;Pop return address (command)
	mov pc,lr 

VM_GetAddress_Branch2:			;Skip 2 bytes
	mov r7,r3
	add r7,r7,#2				;PC+2
	b vm_getaddress_branchgetf
	
	
GetSignExtendedImm8:		;R4=signed result
	.ifdef vm_usevmem
		str lr,[sp,#-4]!
			bl vm_virmemgethlbytea
			mov r4,r0
		ldr lr,[sp],#4
	.else
		ldrb r4,[r3]			;get a byte from the program
	.endif
	add r3,r3,#1				;PC++
	tst r4,#0b10000000
	beq lbl49267
	adrl r10,Mask8bitInv		;Sign Negative
	ldr r10,[r10]
	orr r4,r4,r10
lbl49267:
	mov pc,lr 
	
Mask16bit: .long 0x0000FFFF
Mask16bitInv: .long 0xFFFF0000

Mask8bit: .long 0x000000FF
Mask8bitInv: .long 0xFFFFFF00
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 	Virtual Memory addressing (optional)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;IX = R7 A5 
	;IY = R8 A6
	
	.ifdef vm_usevmem
		
vm_VirMemGetHLByteA:
		str lr,[sp,#-4]!
			str r3,[sp,#-4]!
				bl vm_virmemgetphysical
				ldrb r0,[r3]
			ldr r3,[sp],#4
		ldr lr,[sp],#4
		mov pc,lr 
			
vm_VirMemGetPhysicalIY:		;IY=A6
		mov r3,r8		
		b vm_virmemgetphysical

vm_VirMemGetPhysicalIX:		;IX=A5
		mov r3,r7
		
vm_virmemgetphysical:
		STMFD sp!,{r0,r10,r1}
		and r0,r3,#0xC000		;Keep top 2 bits (banknum)
		
		mov r0,r0,lsr #12		;Switch to bank offset
		
		adrl r10,vm_rambaseAddr
		ldr r10,[r10]			;Get Base of VM Ram
		
		add r0,r0,#vm_rbank0
		ldr r1,[r0,r10]			;Get 32 bit address from VM RAM
		
		mov r0,#0x3FFF			;Add bottom 14 bits
		and r3,r3,r0
		add r3,r3,r1
	
		LDMFD sp!,{r0,r10,r1}
		mov pc,lr 			;Return
		
	.endif
	
	
