ChibiOctave:
	;   	 E     F      G      A      B      C      D   
	.word 0x0000,0x0000,0x0000,0x0020,0x0050,0x0100,0x0200 
	;   	41.2  43.6     49     55   61.7   65.4   73.4
	.word 0x0EAD,0x11b2,0x1743,0x1ccb,0x221a,0x2403,0x2721 ;1
	;  	    82.4  87.3     98    110    123    130    146
	.word 0x2a95,0x2c86,0x317a,0x36d2,0x3c18,0x3ea3,0x4419 ;2
	;		164    174    196    220 	246    261    293
	.word 0x4956,0x4c0f,0x516e,0x56bf,0x5bf8,0x5ea8,0x63fc ;3
	; 	    329    349    392    440    493    523    587
	.word 0x6951,0x6bf2,0x714a,0x76b5,0x7bf9,0x7ead,0x8409 ;4
	;  	    659    698    783    880    987   1046   1174
	.word 0x894e,0x8bf6,0x9142,0x969b,0x9bf8,0x9ea0,0xa14c ;5
	;	   1318   1396   1567   1760   1975   2093   2349
	.word 0xa956,0xabe8,0xb144,0xb68e,0xbbfa,0xbe9f,0xc3eb ;6
	.word 0xC800,0xCC00,0xD000,0xD800,0xE000,0xF000,0xFFFF
	
;r3=volume (0-255) 
;r6=channel num (0-127 unused channels will wrap around) / top bit=noise
;r5=pitch (0-65535) Higher=Higher Pitch

ChibiSoundPro_Set:	
	mov r7,r6
	and r7,r7,#0b00000011
	add r7,r7,#1		;Channel 1-3
	
;Branch based on sound type
	ands r3,r3,#255
	beq ChibiSound_Silent

	
;Volume
	mov r1,r3,lsr #3
	add r1,r1,#0b01100000 ;%1GSAAAAAA G=Gate On/Off S=Smooth Gate A=Amplitude
	add r1,r1,#0x100	;&0100 - &01FF = full amplitude/gate control 
	
;Frequency (Pitch)
	adrl r2,CspMask0xFFFF ;0xFFFF (Bottom 16 bits only)
	ldr r2,[r2]
	and r2,r2,r5
	mov r2,r2,lsr #1
	add r2,r2,#0x100	;&0100 - &7FFF = enhanced pitch control
	
	
;Sound Type?
	tst r6,#0b10000000
	bne ChibiSound_Noise
	
	STMFD sp!,{r1,r2,r3,r5,r6,r7}
		mov r0,r7		;Channel
		mov r1,#10		;Voice (10=Tone)
		SWI 0x40185		;Sound_AttachVoice
	LDMFD sp!,{r1,r2,r3,r5,r6,r7}

ChibiSound_MakeSound:
	mov r0,r7			;R0=Channel
						;R1=Amplitude
						;R2=Pitch
	mov r3,#0xFF		;R3=Duration
	SWI 0x40189			;Sound_Control
	MOV pc,lr
		
ChibiSound_Noise:
	STMFD sp!,{r1,r2,r3,r5,r6,r7}
		mov r0,r7		;Channel
		mov r1,#11		;Voice (11=Noise)
		SWI 0x40185		;Sound_AttachVoice
	LDMFD sp!,{r1,r2,r3,r5,r6,r7}	
	b ChibiSound_MakeSound
		
CspMask0xFFFF:	.long 0xFFFF


ChibiSound_Silent:
	mov r0,r7			;R0=Channel
	mov r1,#0			;R1=Amplitude
						;R2=Pitch
	mov r3,#0xFF		;R3=Duration
	SWI 0x40189			;Sound_Control
	
chibisoundpro_Update:	;For systems that need to do a 'final job'
	MOV pc,lr

	

	
ChibiSoundPro_Init:	
;Set up sound hardware
	mov r0,#4		;Channels
	mov r1,#208		;Samples
	mov r2,#48		;Period
	mov r3,#0		;Handler
	mov r4,#0		;Scheduler
	SWI 0x40140		;Sound_Configure

;Set Channel Panning positions
	mov r0,#1		;Channel
	mov r1,#0		;position (0=Center, 127=Right -127=Left)
	SWI 0x40142		;Sound_Stereo
	
	mov r0,#2		;Channel
	mov r1,#0		;position (0=Center, 127=Right -127=Left)
	SWI 0x40142		;Sound_Stereo
	
	mov r0,#3		;Channel
	mov r1,#0		;position (0=Center, 127=Right -127=Left)
	SWI 0x40142		;Sound_Stereo
	
;Set Global Volume
	mov r0,#127		;Global Volume 127=loudest
	SWI 0x40180		;Sound_Volume
	
	
;Set up Voices
	mov r1,#10		;Voice number (10=Tone)
	SWI 0x40184		;Remove_Voice
	
	adrl r0,VoiceBase ;Voice Pointer
	mov r1,#10		;Voice number
	SWI 0x40183		;Install
	
	mov r1,#11		;Voice number (11=Noise)
	SWI 0x40184		;Remove voice
	
	adrl r0,VoiceBaseNoise ;Voice Pointer
	mov r1,#11		;Voice number
	SWI 0x40183		;Install

	
;Turn on Sound
	mov r0,#2		;Sound On
	SWI 0x40141		;Sound_Enable
	
	mov r0,#2		;Speaker on
	SWI 0x40143		;Sound_Speaker

	MOV pc,lr
	
	
	
;This example is taken from:
; http://www.riscos.com/support/developers/prm/sound.html

VoiceBasea: .long VoiceBase	
	
VoiceBase:
    B       Fill
    B       Fill               ; update entry
    B       GateOn
    B       GateOff
    B       Instance           ; Instantiate entry
    LDMFD   R13!,{PC}          ; Free entry
    LDMFD   R13!,{PC}          ; Initialise
    .long    VoiceName - VoiceBase
 ;
VoiceName: .byte "BeepVoice"
            .byte 0
    .ALIGN 4
 ;**************************************
LogAmpPtr: .long 0
WaveBase:  .long WaveTable
 ;**************************************
Instance: ; any instance must use volume scaled log amp table
    STMFD   R13!,{R0-R4}       ; save registers
    MOV     R0,#0
    MOV     R1,#0
    MOV     R2,#0
    MOV     R3,#0
    MOV     R4,#0
    ;SWI     "XSound_Configure"
	SWI 0x40140
    LDRVC   R0,[R3,#12]        ; get address of volume scaled log amp table
    STRVC   R0,LogAmpPtr       ; and stor
    STRVS   R0, [R13]          ; return error pointe
    LDMFD   R13!,{R0-R4,PC}    ; restore registers and return
 ;**************************************
 ;*  VOICE BUFFER FILL ROUTINES        *
 ;**************************************
 ; on entry:
 ;   r0-r8 available
 ;   r9  is SoundChannelControlBlock pointer
 ;   r10 DMA buffer limit (+1)
 ;   r11 DMA buffer interleave increment
 ;   r12 DMA buffer base pointer
 ;   r13 Sound system Stack with return address and flags
 ;     on top (must LDMFD R13!,{...,pc}
 ; NO r14 - IRQs are enabled and r14 is not usable
GateOn:
    LDR     R0,WaveBase        ; wavetable base
    STR     R0,[R9,#16]        ; set up in SCCB as working register 5
    LDR     R0,LogAmpPtr       ; volume scaled log amp table
    STR     R0,[R9,#20]        ; set up as working register 6
 ;**************************************
Fill:
    LDMIA   R9,{R1-R6}         ; pick up working registers from SCCB
    AND     R1,R1,#0x7F         ; mask R1 so only channel amplitude remain
 ; R1 is amp (0-127)       R2 is pitch phase acc
 ; R3 is timbre phase acc  R4 is duration
 ; R5 is wavetable base    R6 is amp table bas
                               ; move sign bit -> VIDC format lo
    LDRB    R1,[R6,R1,LSL #1]  ; and lookup amp scaled to overall volume 
    MOV     R1,R1,LSR #1       ; move sign bit back again
    RSB     R1,R1,#127         ; make attenuation factor
FillLoop:
    ADD     R2,R2,R2,LSL #16   ; advance waveform phase
    LDRB    R0,[R5,R2,LSR #24] ; get wave sample
    SUBS    R0,R0,R1,LSL #1    ; scale amplitude for overall & channel volumes
    MOVMI   R0,#0              ; and correct underflow
    STRB    R0,[R12],R11       ; generate output sample
    ADD     R2,R2,R2,LSL #16   ; repeated in line four times...
    LDRB    R0,[R5,R2,LSR #24]
    SUBS    R0,R0,R1,LSL #1
    MOVMI   R0,#0
    STRB    R0,[R12],R11
    ADD     R2,R2,R2,LSL #16
    LDRB    R0,[R5,R2,LSR #24]
    SUBS    R0,R0,R1,LSL #1
    MOVMI   R0,#0
    STRB    R0,[R12],R11
    ADD     R2,R2,R2,LSL #16
    LDRB    R0,[R5,R2,LSR #24]
    SUBS    R0,R0,R1,LSL #1
    MOVMI   R0,#0
    STRB    R0,[R12],R11       ; end of repeats..
    CMP     R12,R10            ; check for end of buffer fill
    BLT     FillLoop           ; loop if not
 ; check for end of note
    SUBS    R4,R4,#1           ; decrement centisec count
    STMIB   R9,{R2-R5}         ; save registers to SCC
    MOVPL   R0,#0b00001000      ; voice active if still duration left
    MOVMI   R0,#0b00000010      ; else force flush
    LDMFD   R13!,{PC}          ; return to level 1
 ;**************************************
GateOff:
    MOV     R0,#0
FlushLoop:
    STRB    R0,[R12],R11       ; fill buffer with zeroes
    STRB    R0,[R12],R11
    STRB    R0,[R12],R11
    STRB    R0,[R12],R11
    CMP     R12,R10
    BLT     FlushLoop
 ; CAUSE level 1 TO FLUSH once more
    MOV     R0,#0b00000001      ; set flag to flush one more buffer
    LDMFD   R13!,{PC}          ; return to level 1	
	
WaveTable: 	
		   .space 64,0
		   .space 64,255
		   .space 64,0
		   .space 64,255
		   
		   
;WaveTableNoise:
	; .byte 0xa7,0x00,0xda,0xca,0xc2,0x06,0xd6,0xb8,0x13,0x97,0xca,0x2e,0xb6,0x1e,0x96,0x1a
	; .byte 0xa5,0xb5,0x42,0xe3,0x42,0x2e,0x8b,0x5c,0x45,0x25,0xda,0xf1,0x58,0x21,0x04,0x75
	; .byte 0x62,0x9c,0x82,0x80,0x92,0xd1,0x31,0x3a,0xd5,0x6a,0x5d,0x62,0xc5,0x42,0xe1,0x6c
	; .byte 0x94,0xf2,0xe3,0x47,0xd9,0x46,0x6c,0x79,0x35,0xe0,0x4c,0x3e,0x43,0xc6,0x2c,0x69
	; .byte 0xd7,0x8e,0x21,0xea,0x4c,0xca,0x46,0xd0,0xd2,0x63,0x7b,0x35,0xb7,0x7a,0xdf,0xfe
	; .byte 0x6b,0xb2,0x70,0x87,0x13,0xbd,0x63,0x98,0xbf,0x9a,0xa5,0xbf,0x30,0xc0,0xad,0x23
	; .byte 0x40,0x19,0x7a,0x45,0x6a,0xd0,0x8f,0x94,0xe1,0xa6,0x72,0x81,0xca,0xee,0xbe,0x27
	; .byte 0xbb,0xa8,0xdb,0x84,0x75,0x8f,0x4c,0x3c,0x50,0xef,0x0c,0x2a,0x2e,0x9d,0x1d,0xbb
	; .byte 0x39,0xf1,0xc9,0xa8,0x7d,0x3c,0xc1,0x95,0x69,0x9e,0x77,0xa2,0xda,0x18,0x76,0x7b
	; .byte 0xbe,0x3e,0xff,0x5b,0x00,0xe4,0x11,0xe3,0x34,0x3d,0xd3,0x20,0x9c,0x50,0x57,0xb8
	; .byte 0xc5,0x09,0xb8,0x6d,0xcf,0xb0,0xaa,0xb0,0xf7,0x5f,0xad,0x66,0x8c,0xd0,0x6d,0x76
	; .byte 0x92,0x63,0x40,0x0a,0xc6,0xd6,0x36,0x17,0xa8,0x56,0x24,0xbe,0x63,0x43,0x24,0x8e
	; .byte 0x8e,0xa6,0x88,0x65,0x37,0x09,0x40,0x47,0x91,0x25,0x51,0xc4,0x00,0xd9,0x3c,0x1b
	; .byte 0xe6,0x70,0x25,0x6b,0x99,0xd8,0x56,0xee,0x41,0xb9,0x92,0x18,0x90,0x74,0xf8,0xcf
	; .byte 0xad,0xe1,0x05,0xd6,0xf8,0x3b,0x03,0xf7,0x57,0xe9,0x5a,0xa7,0x72,0xfa,0x0b,0x6e
	; .byte 0x2e,0x00,0xc4,0x7f,0x77,0x90,0x99,0x50,0xc1,0xef,0xbd,0x34,0x74,0x68,0x95,0x8f

WaveTableNoise: 	;WE can Use the source code for noise!	
	.align 4
		 
VoiceBaseNoisea: .long VoiceBaseNoise
	
VoiceBaseNoise:
    B       Fill
    B       Fill               ; update entry
    B       GateOnNoise
    B       GateOff
    B       InstanceNoise           ; Instantiate entry
    LDMFD   R13!,{PC}          ; Free entry
    LDMFD   R13!,{PC}          ; Initialise
    .long    VoiceNameNoise - VoiceBaseNoise
 ;
VoiceNameNoise: .byte "BeepVoice"
            .byte 0
    .ALIGN 4
 ;**************************************
WaveBaseNoise:  .long WaveTableNoise
 ;**************************************
InstanceNoise: ; any instance must use volume scaled log amp table
    STMFD   R13!,{R0-R4}       ; save registers
    MOV     R0,#0
    MOV     R1,#0
    MOV     R2,#0
    MOV     R3,#0
    MOV     R4,#0
    ;SWI     "XSound_Configure"
	SWI 0x40140
    LDRVC   R0,[R3,#12]        ; get address of volume scaled log amp table
    STRVC   R0,LogAmpPtr       ; and stor
    STRVS   R0, [R13]          ; return error pointe
    LDMFD   R13!,{R0-R4,PC}    ; restore registers and return
 ;**************************************
 ;*  VOICE BUFFER FILL ROUTINES        *
 ;**************************************
 ; on entry:
 ;   r0-r8 available
 ;   r9  is SoundChannelControlBlock pointer
 ;   r10 DMA buffer limit (+1)
 ;   r11 DMA buffer interleave increment
 ;   r12 DMA buffer base pointer
 ;   r13 Sound system Stack with return address and flags
 ;     on top (must LDMFD R13!,{...,pc}
 ; NO r14 - IRQs are enabled and r14 is not usable
GateOnNoise:
    LDR     R0,WaveBaseNoise   ; wavetable base
    STR     R0,[R9,#16]        ; set up in SCCB as working register 5
    LDR     R0,LogAmpPtr       ; volume scaled log amp table
    STR     R0,[R9,#20]        ; set up as working register 6
 ;**************************************
	b Fill
		   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
