	.ifndef VM_HostRam
		.equ VM_HostRam,VM_RamBase+0xFF00
	.endif

.equ Cursor_X,VM_HostRam
.equ Cursor_Y,VM_HostRam+1
	.ifndef vm_trap_TableAddr
		.equ vm_trap_TableAddr,VM_HostRam+4
	.endif
.equ vm_remap_TableAddr,VM_HostRam+8


.equ VM_StackTop,0x200



ChibiVM_Init:
	STMFD sp!,{lr}
	
;Clear Low Ram
		adrl r10,vm_rambaseAddr		;Address
		ldr r10,[r10]
		
		mov r2,r10
		mov r1,#512/4
		mov r0,#0
ChibiVM_Init_Clear:	
		str r0,[r2],#4				;Clear first 512 bytes of VM Ram
		subs r1,r1,#1
		bne ChibiVM_Init_Clear
		
;Set defaults
		.ifdef vm_traps
			adrl r0,vm_traps
			adrl r11,vm_trap_TableAddr
			str r0,[r11]
			
			ldrb r1,[r0],#1	 ;R6=Execute address from fixed trap table
			ldrb r2,[r0],#1
			orr r6,r1,r2,lsl #8
			add r6,r6,r10
		.else 
			mov r6,#VM_ProgLoadAddr		;R6 = Execute address
			add r6,r6,r10
		.endif
		
		mov r7,#VM_StackTop				;R7 = Stack Pointer
		add r7,r7,r10
		
		mov r5,#VM_ProgLoadAddr			;R5 = Load Address
		add r5,r5,r10					;     (Rom copy address)
		

;Default Memory Mapping
		mov r0,#0x0000
		add r0,r0,r10				;Add VM Base
		str r0,[r10,#vm_rBank0]
		mov r0,#0x4000
		add r0,r0,r10				;Add VM Base
		str r0,[r10,#vm_rBank1]
		mov r0,#0x8000
		add r0,r0,r10				;Add VM Base
		str r0,[r10,#vm_rBank2]
		mov r0,#0xC000
		add r0,r0,r10				;Add VM Base
		str r0,[r10,#vm_rBank3]
		
		
;Seek a ROM header
		adrl r9,VM_Program	;Start of program ROM
		bl FindHeader 				;Look for a header
				
				
;Update VM state from header (if found, else defaults)
		adrl r10,vm_rambaseAddr		;Address
		ldr r10,[r10]
		
		StoreLEA0 r7,VM_rSP			;Stack Pointer (R7)
		StoreLEA0 r6,VM_rPC			;Program counter (R6)



;Transfer the program to VM RAM
		adrl r3,VM_Program
		mov r1,#VM_Program_End-VM_Program
		
ProgramCopyAgain:	
		ldrb r0,[r3],#1
		strb r0,[r5],#1

		subs r1,r1,#1
		bne ProgramCopyAgain
		
	LDMFD sp!,{pc}

HeaderMagicKey: .byte "!!!!Chibi/VM!!!!"
				
	
FindHeader:			;R9=Search Start  R6=PC R7=SP  R5=Load
	mov r3,r9
	mov r5,#256
MagicKeySearchAgain:	
	adrl r11,HeaderMagicKey		;Key to find
	mov r2,r3
	mov r1,#4					;Longs to scan -1
MagicKeySearching:		
	ldr r0,[r11],#4
	ldr r4,[r2],#4
	cmp r0,r4					;Check this 4 byte block
	bne MagicKeyNG				;Failed? Then give up!
	
	subs r1,r1,#1
	bne MagicKeySearching
	
	b MagicKeyOK
	
MagicKeyNG:
	add r3,r3,#256				;Move to next block
	subs r5,r5,#1				;Try 256 times
	bne MagicKeySearchAgain		;Try again
	mov pc,lr					;Mo Magic Key - RETURN to caller
		
		
MagicKeyOK:		;Key Found! Header is at r3
		ldr r6,[r3,#0x40]		;Defalt PC
		
		mov r5,r6				;Load Address
		add r5,r5,r10
		
		ldr r7,[r3,#0x44]		;Defalt SP

		
		ldr r2,[r3,#0x70]		;Trap table offset
		mov r0,#0
		cmp r2,r0
		beq Header_NoTrapTable
		
		add r2,r2,r9	;For trap table addr relative to prog start
				
		adrl r11,vm_trap_TableAddr
		str r2,[r11]	;Store addr of Trap table in 'ROM'
		
		ldrb r6,[r2]	;First entry is execute address
		ldrb r0,[r2,#1]
		add r6,r6,r0,lsl #8
		
		add r6,r6,r10	;Run address from Trap 0
		
		
		ldr r1,[r3,#0x48]		;Address table of Traps in RAM
								; (Recommended $0100)
		mov r0,#0
		cmp r1,r0
		beq Header_NoTrapTable ;0=Don't copy to RAM
								
		add r1,r1,r10
			
Header_CopyTrapTable:		;Copy ROM trap table to RAM
		ldrb r0,[r2],#1
		strb r0,[r1],#1
		ldrb r4,[r2],#1
		strb r4,[r1],#1
		orrs r4,r4,r0		; 0x0000 = End of table
		bne Header_CopyTrapTable 
		
Header_NoTrapTable:		
		
		
;Copy Address space layout
	mov r1,r3
	add r1,r1,#0x4C		;Copy Default address map
	
	mov r2,r10			;Destination (First bank addr)
	add r2,r2,#vm_rBank0
	mov r4,#32/4		;4 * 4 byte addresses
	
Header_CopyAddressMap:	
	ldr r0,[r1],#4
	str r0,[r2],#4
	subs r4,r4,#1
	bne Header_CopyAddressMap
	
	
;Address Remap table address

	ldr r1,[r3,#0x74]	;Address Remap Table
	adrl r11,Addr_vm_remap_TableAddr
	ldr r11,[r11]
	str r1,[r11]		;Store Table address in VM settings
		
	mov pc,lr			;RETURN (All done)

Addr_vm_remap_TableAddr:	.long vm_remap_TableAddr



