;Text after semicolon is comments, you don't need to type it in

	.org  0x08000000   			;GBA Rom Base
;Header
	b	ProgramStart			;Jump to our program
	.space 178					;Logo (omitted) + Program name
	.byte 0x96					;Fixed value
	.space 49					;Dummy Header 
	
ProgramStart:
	adr r0,ThumbTest			;Address of thumb code
	add r0,r0,#1				;Bit 0 set = THUMB ON!
	bx r0						;Branch and exchange

	.thumb						;We are now in thumb mode
ThumbTest:
	ldr r1,SPAddress			;Init Stack Pointer SP=0x03000000
	mov sp,r1
	
	ldr r4,ScreenRegs  			;0x04000000 DISPCNT -LCD Control
	ldr r2,ScreenSetting   		;0x403 4= Layer 2 on / 3= ScreenMode 3
	str	r2,[r4,#0]         	

	mov r6,#10					;Xpos
	mov r7,#10					;Ypos
	
	bl ShowSprite				;Show the sprite starting position
	
InfLoop:
	ldr r3,JoyPort				;Read GBA joypad (0x4000130)
	ldrh r0,[r3,#0]		
	         ;------lrDULRSsBA
	mov r1,#0b0000000011110000
	and r0,r1
	cmp r0,r1					;Wait for a key to be pressed
	beq InfLoop
	
	bl ShowSprite				;Remove the old sprite
	         ;------lrDULRSsBA
	mov r1,#0b0000000001000000
	tst r0,r1					;Check Up Key
	bne JoyNotUp
	cmp r7,#0					;At top of screen?
	beq JoyNotUp
	sub r7,#1					;Move Up
JoyNotUp:	
			 ;------lrDULRSsBA
	mov r1,#0b0000000010000000
	tst r0,r1					;Check Up Down Key
	bne JoyNotDown
	cmp r7,#19					;At bottom of screen?
	beq JoyNotDown
	add r7,#1					;Move Down
JoyNotDown:	
	         ;------lrDULRSsBA
	mov r1,#0b0000000000100000
	tst r0,r1					;Check Left Key
	bne JoyNotLeft	
	cmp r6,#0					;At Left of screen?
	beq JoyNotLeft
	sub r6,#1					;Move Left
JoyNotLeft:	
			 ;------lrDULRSsBA
	mov r1,#0b0000000000010000
	tst r0,r1					;Check Right Key
	bne JoyNotRight
	cmp r6,#29					;At Right of screen?
	beq JoyNotRight
	add r6,#1					;Move Right
JoyNotRight:	

	bl ShowSprite				;Show the new sprite position

	ldr r0,DelayLength			;Delay Loop
Delay:	
	sub r0,#1
	bne Delay
	b InfLoop					;Repeat
		
ShowSprite:			;X,Y = R6,R7
	push {r0-r7, lr}	
		ldr r3,ScreenRam 		;0x06000000 - VRAM Dest
		
		mov r1,#16				;Sprite is 8px, 2 bytes per pixel
		mul r6,r1
		add r3,r3,r6			;Xpos *16
	
		ldr r1,LineSize			;240*8*2 
		mul r7,r1
		add r3,r3,r7			;Ypos * 240*8*2
		
		ldr r4,SpriteAddress 	;Sprite Source Address

		ldr r0,LineSize2		;480-16
		mov r8,r0				;Amount to add per Y-line
		
		mov r6,#8				;8 Y-lines 
Sprite_NextLine:
		mov r7,#8 				;8 pixels per X-line
Sprite_NextByte:
		ldrh r2,[r4,#0]			;Load Bitmap Pixel
		ldrh r1,[r3,#0]			;Load Screen Pixel
		eor r1,r2				;Eor two together 
		strh r1,[r3,#0]			;Write update pixel to screen

		add r4,r4,#2			;Move to next Src pixel
		add r3,r3,#2			;Move to next VRAM pixel
		sub r7,r7,#1
		bne Sprite_NextByte		;Next X pixel
			
		add r3,r8 				;Move Down a line (240 pixels *2 bytes) 
		sub r6,r6,#1								;-1 block (16 px)
		bne Sprite_NextLine		;Next Y line
	pop {r0-r7, pc}
	  
	.align 4
SPAddress:
	.long 0x03000000			;Stack pointer address
ScreenRegs:
	.long 0x04000000			;Video Register base
ScreenSetting:
	.long 0x403					;4= Layer 2 on / 3= ScreenMode 3
ScreenRam:
	.long 0x06000000			;Vram Base
LineSize:
	.long 240*8*2				;Width of one char line (8 pixels)
LineSize2:
	.long 480-16				;pixel width of one line - 1 char
SpriteAddress:
	.long SpriteTest			;Address of Sprite
JoyPort:
	.long 0x4000130
DelayLength: 
	.long 0x8FFF

SpriteTest:		;Smiley ABBBBBGGGGGRRRRR - A=Alpha B=Blue G=Green R=Red
	.word 0x8000,0x8000,0x83FF,0x83FF,0x83FF,0x83FF,0x8000,0x8000 ;  0
	.word 0x8000,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x8000 ;  1
	.word 0x83FF,0x83FF,0x801F,0x83FF,0x83FF,0x801F,0x83FF,0x83FF ;  2
	.word 0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF ;  3
	.word 0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF,0x83FF ;  4
	.word 0x83FF,0x83FF,0xFFE0,0x83FF,0x83FF,0xFFE0,0x83FF,0x83FF ;  5
	.word 0x8000,0x83FF,0x83FF,0xFFE0,0xFFE0,0x83FF,0x83FF,0x8000 ;  6
	.word 0x8000,0x8000,0x83FF,0x83FF,0x83FF,0x83FF,0x8000,0x8000 ;  7

	
	