	
	.org  0x08000 			;Program Start
	b RiscOS_Start			;Jump to the start of our program
	.space 1024				;Space for the stack
Stack:

RiscOS_Start:
	adrl sp,stack			;Load the stack pointer 
	
	
;Set up screen
	mov r0,#22				;VDU 22 - Selects screen mode 
	swi 0x00				;print char R0
	mov r0,#9				;Screen mode 9 = 320 × 256 @ 16 colors
							;Screen mode 13 = 320 × 256 @ 256 colors
	swi 0x00				;print char R0
	SWI 0x36 				;Remove Cursor
  
;Get Screen Base Addr
	adrl r0,GetVars			;Vars We want
	adrl r1,ReturnVars		;Area to store returned data
	swi 0x31				;OS_ReadVduVariables
		
;Set up the palette
	adr r10,InitPalette		;Source Palette
	mov r0,#0
InitPaletteAgain:
	ldr r1,[r10],#2
	bl SetPalette			;Palette Conversion
	add r0,r0,#1
	cmp r0,#16
	bne InitPaletteAgain


	
;Sprite Pos	
	mov r8,#10			;Xpos
	mov r9,#10			;Ypos
	
	bl ShowSprite		;Show starting position of sprite
		
InfLoop:
	bl ShowSprite		;Remove the old sprite
	
	mov r0,#129			;Test a key
	mov r1,#255^57		;Key 57=UP (255 xor key)
	mov r2,#255			;255=No Wait
	SWI 0x6				;OSByte
	cmp r1,#255			;255=Pressed 0=Unpressed
	bne JoyNotUp
	cmp.b r9,#0			;at top of screen?
	beq JoyNotUp
	sub.b r9,r9,#1		;Move Up
JoyNotUp:	
	mov r0,#129			;Test a key
	mov r1,#255^41		;Key 41=DOWN (255 xor key)
	mov r2,#255			;255=No Wait
	SWI 0x6				;OSByte
	cmp r1,#255			;255=Pressed 0=Unpressed
	bne JoyNotDown
	cmp.b r9,#256-8		;at bottom of screen?
	beq JoyNotDown
	add.b r9,r9,#1		;Move Down
JoyNotDown:	
	mov r0,#129			;Test a key
	mov r1,#255^25		;Key 25=LEFT (255 xor key)
	mov r2,#255			;255=No Wait
	SWI 0x6				;OSByte
	cmp r1,#255			;255=Pressed 0=Unpressed
	bne JoyNotLeft
	cmp.b r8,#0			;at left of screen?
	beq JoyNotLeft
	sub.b r8,r8,#1		;Move Left
JoyNotLeft:	
	mov r0,#129			;Test a key
	mov r1,#255^121		;Key 121=RIGHT (255 xor key)
	mov r2,#255			;255=No Wait
	SWI 0x6				;OSByte
	cmp r1,#255			;255=Pressed 0=Unpressed
	bne JoyNotRight
	cmp.b r8,#160-4		;at right of screen?
	beq JoyNotRight
	add.b r8,r8,#1		;Move Right
JoyNotRight:	

	bl ShowSprite		;Show the new sprite position

	mov r0,#0x60000		;Delay Loop
Delay:	
	subs r0,r0,#1
	bne Delay
	b InfLoop			;Repeat.
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	

ShowSprite:		;R8=X R9=Y

	STMFD sp!,{r0-r12, lr}		
		;ScreenAddr = ScreenBase + (Ypos*160) + Xpos
		adrl r1,ScreenBase		;get ScreenAddr
		ldr r10,[r1]		
		mov r1,#160				;Ypos *160
		mla r2,r1,r9,r10		;Multiply Ypos * 16 and add base
		add r10,r2,r8			;+Xpos
		
		ldr r1,SpriteAddress
		mov r6,#8				;Height
		
Sprite_NextLine:	
		mov r5,#8/2				;Width
		STMFD sp!,{r10}
Sprite_NextByte:
			ldrB r0,[r1],#1		;Copy a byte to the screen
			ldrB r2,[r10]
			eor r0,r0,r2		;Xor Byte from screen 
			strB r0,[r10],#1
			
			subs r5,r5,#1
			bne Sprite_NextByte	;Next Pixel
		LDMFD sp!,{r10}		
		
		add r10,r10,#160		;Move down a line
		
		subs r6,r6,#1
		bne Sprite_NextLine		;Next Line
	LDMFD sp!,{r0-r12, pc}	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetPalette:		;R0= Palette Number R1= 0x-----GRB
	STMFD sp!,{r0-r12, lr}	
		mov r11,r0			;Palette num
		
		mov r0,#19			;VDU 19	-Set palette
;VDU 19,logical colour,mode,red,green,blue
		swi 0x00			;print char R0
		
		mov r0,r11			;palette number
		swi 0x00			;print char R0
		
		mov r0,#16			;NoFlash
		swi 0x00			;print char R0
		
		mov r2,r1			;Red
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;R (top 4 bit 00-F0)
		swi 0x00			;print char R0
		
		mov r2,r1,ROR #4	;Green
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;G (top 4 bit 00-F0)
		swi 0x00			;print char R0
		
		mov r2,r1,LSL #4	;Blue
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;B (top 4 bit 00-F0)
		swi 0x00			;print char R0
	LDMFD sp!,{r0-r12, pc}	
	
	
InitPalette:
		  ;-GRB
	.word 0x008;0  
	.word 0xFF0;1  
	.word 0xF0F;2  
	.word 0x0F0;3  
	.word 0xFF0;4  
	.word 0xFF0;5  
	.word 0xFF0;6  
	.word 0xFF0;7  
	.word 0xFF0;8  
	.word 0xFF0;9  
	.word 0xFF0;A  
	.word 0xFF0;B  
	.word 0xFF0;C  
	.word 0xFF0;D  
	.word 0xFF0;E  
	.word 0xFF0;F  
	

	
	
	
	
GetVars:
	.long 148	;ScreenStart
	.long 149	;DisplayStart
	.long 150	;TotalScreenSize
	.long -1	;List End

ReturnVars:
ScreenBase:		.long 0		;ScreenStart
				.long 0		;DisplayStart
				.long 0		;TotalScreenSize
				

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
SpriteAddress:
	.long Sprite
		
Sprite:	;Smiley face - one nibble per color (Nibbles swapped)
		;   RL   RL   RL   RL  - Right / Left
    .byte 0x00,0x11,0x11,0x00     ;  0
    .byte 0x10,0x11,0x11,0x01     ;  1
    .byte 0x11,0x13,0x31,0x11     ;  2
    .byte 0x11,0x11,0x11,0x11     ;  3
    .byte 0x11,0x11,0x11,0x11     ;  4
    .byte 0x11,0x12,0x21,0x11     ;  5
    .byte 0x10,0x21,0x12,0x01     ;  6
    .byte 0x00,0x11,0x11,0x00     ;  7

	
	
	