	
	.org  0x08000 			;Program Start
	b RiscOS_Start			;Jump to the start of our program
	
Score:			.long 0			;Score BCD
Hiscore:		.long 0			;HiScore BCD
SoundTimeOut:	.byte 0			;Sound Time
Lives:			.byte 0			;Lives
	
	.align 4
randomseed:		.long 0			;2 Bytes for random numbers

EnemyData:		.byte 0,0,0,0,0,0,0,0	
.equ EnemyData_X,0				;Xpos
.equ EnemyData_Y,1				;Ypos
.equ EnemyData_Spr,2			;Frame 0-1
.equ EnemyData_Sca,3			;Scale 16-64
.equ EnemyData_Speed,4			;Speed
.equ EnemyData_SpeedB,5			;time till next move
.equ EnemyData_Tick,6			;time till next frame
	
RamArea:
	.byte 0,0
	.equ CursorX,0				;offset in ramarea
	.equ CursorY,1				;Text Cursor Position

	
	.align 4
	.space 1024				;Space for the stack
Stack:



RiscOS_Start:
	adrl sp,stack			;Load the stack pointer 
		
;Set up screen
	mov r0,#22				;VDU 22 - Selects screen mode 
	swi 0x00				;print char R0
	mov r0,#9				;Screen mode 9 = 320 × 256 @ 16 colors
							;Screen mode 13 = 320 × 256 @ 256 colors
	swi 0x00				;print char R0
	SWI 0x36 				;Remove Cursor
  
;Get Screen Base Addr
	adrl r0,GetVars			;Vars We want
	adrl r1,ReturnVars		;Area to store returned data
	swi 0x31				;OS_ReadVduVariables
		
;Set up the palette
	adr r10,InitPalette		;Source Palette
	mov r0,#0
InitPaletteAgain:
	ldr r1,[r10],#2
	bl SetPalette			;Palette Conversion
	add r0,r0,#1
	cmp r0,#16
	bne InitPaletteAgain


	
	
ShowTitle:
	bl WaitForRelease			;Wait for Fire to be released

	bl Cls						;Clear Screen
	
	mov r11,#SoundTimeOut	
	mov r0,#0
	strb r0,[r11]				;Silence sound
		
	mov r1,#20-5
	mov r2,#10
	bl Locate	;Locate X,Y=R1,R2
	ldr r1,atxtSuckShoot
	bl PrintString				;Show Title
	
	mov r1,#20-4
	mov r2,#20
	bl Locate	;Locate X,Y=R1,R2
	ldr r1,atxtHiscore
	bl PrintString				;Show Hiscore text
	
	mov r1,#20-4
	mov r2,#21
	bl Locate	;Locate X,Y=R1,R2
	mov r11,#HiScore			;Show Hiscore
	ldr r1,[r11]	
	bl BCD_Show					;Show BCD in R1
	
	mov r11,#EnemyData
	
	mov r0,#80					;Settings for bat sprite
	strb r0,[r11,#EnemyData_X]
	mov r0,#128
	strb r0,[r11,#EnemyData_Y]
	mov r0,#1
	strb r0,[r11,#EnemyData_Spr]
	mov r0,#48
	strb r0,[r11,#EnemyData_Sca]
	
	
TitleScreen:	
	mov r11,#EnemyData
	
	ldrb r0,[r11,#EnemyData_Spr]	
	eor r0,r0,#1				;Toggle bat frame
	strb r0,[r11,#EnemyData_Spr]

	bl ShowEnemy				;Show The Bat

	mov r0,#0x1000000				;Wait a bit!
	bl DoPause
	
	bl ShowEnemy				;Remove The Bat
	
	bl ReadJoystick				;Read in the Joystick	%---1UDLR
		    ; SSBARLDU
	tst r0,#0b00010000
	bne TitleScreen				;Repeat until fire pressed
	b Gameplay

atxtSuckShoot:
	.long txtSuckShoot			;Pointers to text strings
atxtHiscore:
	.long txtHiscore

txtSuckShoot:
	 .byte "SUCK SHOOT!",255
txtHiscore:
	 .byte "HISCORE:",255
	.align 4
	
	
Gameplay:
	bl Cls						;Clear Screen
	
	mov r11,#EnemyData
	
	mov r0,#32					;Enemy Speed
	strb r0,[r11,#EnemyData_Speed]
	
	mov r11,#Score
	mov r0,#0
	str r0,[r11]				;Write 4 zeros to score
	
	mov r11,#Lives
	mov r0,#4
	strb r0,[r11]				;Player lives
	
	bl RandomizeEnemy
	
	bl UpdateScore				;Show Score/lives text
	
	mov r8,#12					;Player Xpos
	mov r9,#10					;Player Ypos
	
	STMFD sp!,{r8-r9}
		bl ShowPlayer			;Show the new sprite position
		bl ShowEnemy			;Show the enemy sprite
	LDMFD sp!,{r8-r9}		
	
	
	
InfLoop:
	mov r10,#SoundTimeOut		;Time till silenced
	ldrb r0,[r10]
	cmp r0,#0
	beq ChibiSoundUpdated
	subs r0,r0,#1
	bne ChibiSoundUpdated
	bl ChibiSound				;Mute Sound
ChibiSoundUpdated:
	strb r0,[r10]
	
	
	
	bl ReadJoystick				;Read in the Joystick	%--21UDLR
	
	STMFD sp!,{r0,r8-r9}
		bl ShowPlayer			;Remove the old player
	LDMFD sp!,{r0,r8-r9}		
	
	mov r7,#2					;Move Speed Slow
	        ; SSBARLDU
	tst r0,#0b00100000			;Fire 2
	bne NoFire2
	mov r7,#4					;Move Speed Fast
NoFire2:	

	        ; SSBARLDU
	tst r0,#0b00000001
	bne JoyNotUp
	cmp r9,#12
	bcc JoyNotUp
	sub r9,r9,r7				;Move Up
JoyNotUp:	
	        ; SSBARLDU
	tst r0,#0b00000010
	bne JoyNotDown
	cmp r9,#256-12
	bcs JoyNotDown
	add r9,r9,r7				;Move Down
JoyNotDown:	
	        ; SSBARLDU
	tst r0,#0b00000100
	bne JoyNotLeft
	cmp r8,#12
	bcc JoyNotLeft
	sub r8,r8,r7				;Move Left
JoyNotLeft:	
	        ; SSBARLDU
	tst r0,#0b00001000
	bne JoyNotRight
	cmp r8,#160-6
	bcs JoyNotRight
	add r8,r8,r7				;Move Right
JoyNotRight:	
	STMFD sp!,{r0,r8-r9}
		bl ShowPlayer			;Show the new sprite position
		bl ShowEnemy			;Remove old enemy
	LDMFD sp!,{r0,r8-r9}		
	
	
	
	
	STMFD sp!,{r8-r9}
			    ; SSBARLDU
		tst r0,#0b00010000
		bne NoFire1				;See if player shot
		
		mov r11,#SoundTimeOut
		ldrb r0,[r11]
		cmp r0,#0
		bne FireNosound			;Done make fire sound if still firing
		
		mov r0,#0b11100000
		bl ChibiSound			;Make sound R0
FireNosound:		

		mov r11,#SoundTimeOut
		mov r0,#4
		strb r0,[r11]			;Set Sound Time
	
		mov r2,r8
		mov r5,r9
		
		mov r11,#EnemyData
		ldrb r1,[r11,#EnemyData_X]	;XY of enemy
		ldrb r4,[r11,#EnemyData_Y]
				
		mov r3,#8				;Width of Range
		mov r6,#8				;Height of Range
	
		bl rangetest			;See if object XY pos r2,r5 hits 
		beq NoHit				;object r1,r4 in range r3,r6
		
		ldrb r0,[r11,#EnemyData_Speed]
		add r0,r0,#1			;Speed up next enemy
		strb r0,[r11,#EnemyData_Speed]
		
		mov r0,#0b11000001
		bl ChibiSound			;Make sound A
				
		mov r10,#Score
		ldr r1,[r10]			;Get Current Score
		mov r2,#5
		
		bl BCD_Add				;R1=R1+R2
		str r1,[r10]
		
		bl UpdateScore			;Show Score/lives text

		bl RandomizeEnemy		;Hit object, so re-randomize
NoHit:
NoFire1:	

		mov r11,#EnemyData
		ldrb r0,[r11,#EnemyData_Tick]
		add r0,r0,#16			;Update Bat Frame Tick
		strb r0,[r11,#EnemyData_Tick]
		ands r0,r0,#0xF00
		beq NoEnemyHit			;If not overflowed no move yet
		
		mov r11,#EnemyData
		ldrb r0,[r11,#EnemyData_Spr]
		eor r0,r0,#1			;Toggle Bat frame
		strb r0,[r11,#EnemyData_Spr]

		
		
		ldrb r0,[r11,#EnemyData_SpeedB]
		ldrb r1,[r11,#EnemyData_Speed]
		add r0,r0,r1			;Update Bat Time
		strb r0,[r11,#EnemyData_SpeedB]
		
		ands r0,r0,#0x0F00		
		beq NoEnemyHit
		
		ldrb r0,[r11,#EnemyData_Sca]
		adds r0,r0,#8			;Make Bat Bigger
		strb r0,[r11,#EnemyData_Sca]
		
		cmp r0,#64				;Biggest?
		bcc NoEnemyHit
			bl RandomizeEnemy	;Bat bit us!
			
			mov r11,#Lives
			ldrb r0,[r11]	
			subs r0,r0,#1		;Remove 1 life
			strb r0,[r11]	
			beq GameOver		;Player dead?
			
			mov r11,#SoundTimeOut
			mov r0,#4
			strb r0,[r11]		;Set Sound Time
			
			mov r0,#0b01000001
			bl ChibiSound		;Make Bite sound
		
			bl UpdateScore		;Show Score/lives text
NoEnemyHit:		
		bl ShowEnemy			;Show Bat again
	LDMFD sp!,{r8-r9}		
	
	
	
	mov r0,#0x100000			;Delay Loop
	bl DoPause
	
	b InfLoop					;Repeat.

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	

GameOver:
	bl WaitForRelease			;Wait for Fire to be released
	
	mov r0,#0
	bl ChibiSound				;Silence sound

	bl Cls						;Clear Screen
	
	mov r1,#10
	mov r2,#4
	bl Locate	;Locate X,Y=R1,R2	
	ldr r1,atxtGameOver			;Show 'Game Over'
	bl PrintString

	mov r10,#Score
	ldr r1,[r10]				;Current Score
	mov r11,#HiScore
	ldr r0,[r11]				;High score
	
	cmp r1,r0
	bls GameOver_NoScore		;New score lower

	str r1,[r11]				;Update the highscore
	
	mov r1,#9
	mov r2,#15
	bl Locate	;Locate X,Y=R1,R2	
	ldr r1,atxtGotHiScore		;Show 'New Highscore'
	bl PrintString
	
GameOver_Wait:
	bl ReadJoystick		;Read in the Joystick	%---1UDLR
		    ; SSBARLDU
	tst r0,#0b00010000
	bne GameOver_Wait			;Repeat until fire pressed
	b ShowTitle
	
GameOver_NoScore:	
	mov r1,#10
	mov r2,#15
	bl Locate	;Locate X,Y=R1,R2	
	ldr r1,atxtNoHiScore		;Show 'You Suck!'
	bl PrintString
	b GameOver_Wait
	
atxtGameOver:
	.long txtGameOver
atxtGotHiScore:
	.long txtGotHiScore
atxtNoHiScore:
	.long txtNoHiScore
	
txtGameOver:
	.byte "GAME OVER",255
txtGotHiScore:	
	.byte "NEW HISCORE",255
txtNoHiScore:				
	.byte "YOU SUCK!",255	

	
	
UpdateScore:
	STMFD sp!,{lr}
		mov r1,#0
		mov r2,#0
		bl Locate	;Locate X,Y=R1,R2
		mov r11,#Score
		ldr r1,[r11]	
		bl BCD_Show				;Show Score TopLeft
		
		mov r1,#38
		mov r2,#0
		bl Locate	;Locate X,Y=R1,R2
		mov r11,#Lives
		ldrb r1,[r11]	
		mov r1,r1,asl #24
		mov r4,#2				;Show Lives Top Right
		bl ShowBCDalt
	LDMFD sp!,{pc}
	
DoPause:
	subs r0,r0,#1
	bne DoPause
	MOV pc,lr

	
Cls:
		adrl r1,ScreenBase		;get VFAM base
		ldr r0,[r1]		
		mov r1, #0				;Fill Word
		mov r2, #320*256/8		;Words to fill
FillScreenLoop:
		str r1, [r0],#4			;Store+inc 4 bytes
		subs r2, r2, #1		
		bne FillScreenLoop	
	MOV pc,lr

	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
;Xor Sprite, drawing twice will remove sprite from screen.
ShowEnemy:
	mov r11,#EnemyData
	ldrb r8,[r11,#EnemyData_X]
	ldrb r9,[r11,#EnemyData_Y]
	ldrb r10,[r11,#EnemyData_Spr]
	and r10,r10,#1				;2 frames of animation
	add r10,r10,#1				;Enemy sprites are no 1-7
	
	ldrb r0,[r11,#EnemyData_Sca]
	eor r0,r0,#0b00110000
	and r0,r0,#0b00110000		;3 sizes of bat
	add r10,r10,r0,lsr #3
	b ShowSprite
		
ShowPlayer:
	mov r10,#0					;Player cursor is sprite 0
	
ShowSprite:
	mov r8,r8,lsl #1			;Our Xpos is in pairs of pixels 

	ldr r11,aSpriteInfo
	add r11,r11,r10,lsl #3
	
	ldrb r0,[r11,#4]			;Width
	mov r0,r0,lsr #1
	sub r8,r8,r0
	
	ldrb r0,[r11,#5]			;Height
	mov r0,r0,lsr #1
	sub r9,r9,r0
	
	
	
	adrl r1,ScreenBase			;get ScreenAddr
	ldr r10,[r1]		
	mov r1,#160					;Ypos *160
	mla r2,r1,r9,r10			;Multiply Ypos * 16 and add base
	add r10,r2,r8,lsr #1		;+Xpos in bytes
	
	
	ldr r1,[r11]				;Sprite source Address
	
	ldrb r6,[r11,#5]			;Height
Sprite_NextLine:	
	ldrb r5,[r11,#4]			;Width (in pixels)

	STMFD sp!,{r10}
Sprite_NextByte:
		ldrB r3,[r1],#1			;Must write 16/32bit per VRAM write 
		ldrB r2,[r10]
		eor r3,r3,r2			;Eor Word from screen
		strB r3,[r10],#1
		
		subs r5,r5,#2			;X Loop
		bne Sprite_NextByte
	LDMFD sp!,{r10}		
	add r10,r10,#160			;240 - 2 bytes per pixel
	subs r6,r6,#1
	bne Sprite_NextLine			;Y loop
	mov pc,lr
	

	
aSpriteInfo:
	.long SpriteInfo
	
SpriteInfo:
    .long 0x0000/2 + SpriteBase           ;SpriteAddr 0
     .byte 16,16                                    ;XY 
	 .byte 0,0
    .long 0x0100/2 + SpriteBase           ;SpriteAddr 1
     .byte 32,24                                     ;XY 
	 .byte 0,0
    .long 0x0400/2 + SpriteBase           ;SpriteAddr 2
     .byte 32,32                                     ;XY 
	 .byte 0,0
    .long 0x0800/2 + SpriteBase           ;SpriteAddr 3
     .byte 24,16                                     ;XY 
	 .byte 0,0
    .long 0x0980/2 + SpriteBase           ;SpriteAddr 4
     .byte 24,24                                     ;XY 
	 .byte 0,0
    .long 0x0BC0/2 + SpriteBase           ;SpriteAddr 5
     .byte 16,8                                      ;XY 
	 .byte 0,0
    .long 0x0C40/2 + SpriteBase           ;SpriteAddr 6
     .byte 16,16                                     ;XY 
	.byte 0,0
	
SpriteBase: 	;16 color sprites
	.incbin "\ResALL\SuckShoot\ROS_SuckShoot.RAW"

	
	.align 4	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetPalette:		;R0= Palette Number R1= 0x-----GRB
	STMFD sp!,{r0-r12, lr}	
		mov r11,r0			;Palette num
		
		mov r0,#19			;VDU 19	-Set palette
;VDU 19,logical colour,mode,red,green,blue
		swi 0x00			;print char R0
		
		mov r0,r11			;palette number
		swi 0x00			;print char R0
		
		mov r0,#16			;NoFlash
		swi 0x00			;print char R0
		
		mov r2,r1			;Red
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;R (top 4 bit 00-F0)
		swi 0x00			;print char R0
		
		mov r2,r1,ROR #4	;Green
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;G (top 4 bit 00-F0)
		swi 0x00			;print char R0
		
		mov r2,r1,LSL #4	;Blue
		and r2,r2,#0xF0 	;Nibble Mask
		mov r0,r2			;B (top 4 bit 00-F0)
		swi 0x00			;print char R0
	LDMFD sp!,{r0-r12, pc}	
	
	
InitPalette:
		  ;-GRB
    .word 0x0000 ;0  -GRB
    .word 0x000A ;1  -GRB
    .word 0x0A10 ;2  -GRB
    .word 0x0A0A ;3  -GRB
    .word 0x00A0 ;4  -GRB
    .word 0x00AA ;5  -GRB
    .word 0x05A0 ;6  -GRB
    .word 0x0AAA ;7  -GRB
    .word 0x0555 ;8  -GRB
    .word 0x055F ;9  -GRB
    .word 0x0F55 ;10  -GRB
    .word 0x0F5F ;11  -GRB
    .word 0x05F5 ;12  -GRB
    .word 0x05FF ;13  -GRB
    .word 0x0FF5 ;14  -GRB
    .word 0x0FFF ;15  -GRB


	
	
	
	
GetVars:
	.long 148	;ScreenStart
	.long 149	;DisplayStart
	.long 150	;TotalScreenSize
	.long -1	;List End

ReturnVars:
ScreenBase:		.long 0		;ScreenStart
				.long 0		;DisplayStart
				.long 0		;TotalScreenSize
				


dorangedrandom:		;return a value in R0 between R1 and R4
	STMFD sp!,{lr}
dorangedrandomB:			
		bl dorandom
		cmp	r0,r1
		blt dorangedrandomB		;Below R1
		cmp	r0,r4
		bgt dorangedrandomB		;Above R4
	LDMFD sp!,{pc}
	
WaitForRelease:	
	STMFD sp!,{lr}
		bl ReadJoystick
				; SSBARLDU
		tst r0,#0b00010000
		beq WaitForRelease			;Repeat until fire released
	LDMFD sp!,{pc}
	

RandomizeEnemy:	
	STMFD sp!,{lr}
		mov r11,#EnemyData
		
		mov r1,#16
		mov r4,#160-8
		bl dorangedrandom
		strb r0,[r11,#EnemyData_X]		;Random Xpos
	
		mov r1,#16
		mov r4,#256-16
		bl dorangedrandom
		strb r0,[r11,#EnemyData_Y]		;Random Ypos
		
		
		mov r0,#16
		strb r0,[r11,#EnemyData_Sca]	;Scale=0
		
		mov r0,#0
		strb r0,[r11,#EnemyData_SpeedB]	;Speed=0
		strb r0,[r11,#EnemyData_Spr]	;Frame=0
	LDMFD sp!,{pc}
		
	
;Random number Lookup tables
Randoms1:
	.byte 0x0A,0x9F,0xF0,0x1B,0x69,0x3D,0xE8,0x52,0xC6,0x41,0xB7,0x74,0x23,0xAC,0x8E,0xD5
Randoms2:
	.byte 0x9C,0xEE,0xB5,0xCA,0xAF,0xF0,0xdb,0x69,0x3D,0x58,0x22,0x06,0x41,0x17,0x74,0x83
	
	.include "\SrcAll\V1_Random.asm"
	.include "\SrcAll\V1_BCD.asm"
;	.include "\SrcALL\V1_ChibiSound.asm"	


	
;See if object XY pos R2,R5 hits object R1,R4 in range R3,R6
	;Return R0=1 Collision... R0!=1 means no collision
rangetest:			
	STMFD sp!,{lr}	;X Axis Check
	
		mov r0,r1 			;Pos1 Xpos
		subs r0,r0,r3		;Shift Pos1 by 'range' to the Left
		blt rangetestb		;<0
		cmp r2,r0			;Does it match Pos2?
		blt rangetestoutofrange
		
rangetestb:
		add r0,r0,r3,asl #1	;Shift Pos1 by 'range' to the Right
		cmp r0,#255
		bgt rangetestd		;>255
		cmp r2,r0			;Does it match Pos2?
		bgt rangetestoutofrange
		
rangetestd:			;Y Axis Check

		mov r0,r4 			;Pos1 Ypos
		subs r0,r0,r6		;Shift Pos1 by 'range' Up
		blt rangetestc		;<0
		cmp r5,r0 			;Does it match Pos2?
		blt rangetestoutofrange
		
rangetestc:
		add r0,r0,r6,asl #1	;Shift Pos1 by 'range' Down
		cmp r0,#255
		bgt rangeteste		;>255
		cmp r5,r0			;Does it match Pos2?
		bgt rangetestoutofrange			
rangeteste:
		movs r0,#1			;1=Collided
	LDMFD sp!,{pc}
	
rangetestoutofrange:		
		movs r0,#0			;0=No Collision
	LDMFD sp!,{pc}
					
				
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
SpriteAddress:
	.long Sprite
		
Sprite:	;Smiley face - one nibble per color (Nibbles swapped)
	.incbin "\ResALL\SuckShoot\ROS_SuckShoot.RAW"

PrintString:
	STMFD sp!,{r0-r12, lr}
PrintStringAgain:
		ldrB r0,[r1],#1
		cmps r0,#255
		beq PrintStringDone
		bl printchar 
		b PrintStringAgain
PrintStringDone:
	LDMFD sp!,{r0-r12, pc}


PrintChar:
	STMFD sp!,{r0-r12, lr}
		mov r3,#RamArea
		ldrB r4,[r3,#CursorX]	;X
		ldrB r5,[r3,#CursorY]	;Y
				
		adrl r1,ScreenBase
		ldr r3,[r1]			;Get Screen Base
		
		mov r6,#4			;Xpos (4 bytes per Char)
		mul r2,r4,r6
		add r3,r3,r2
		
		mov r4,#160*8		;Ypos (160 bytes per line - 8 lines per char)
		mul r2,r5,r4
		add r3,r3,r2
		
		adrl r4,BitmapFont 	;Address of our font
		sub r0,r0,#32		;No chars <32 (first Char = space)
		add r4,r4,r0,asl #3 ;Char * 8 - offset to 
		
		mov r1,#8 ; 48 bytes per row = 12 words
NextFontLine:

		mov r9,#0b00000001
		ldrb r8,[r4]	;Load line of 1bpp font
		add r4,r4,#1
		
		mov r2,#0
		;We're going to move a bit into each nibble of R2
		
		and r10,r9,r8			;-------X
		orr r2,r2,r10
		
		and r10,r9,r8,ror #1	;------X-
		orr r2,r2,r10,ror #4*1
		
		and r10,r9,r8,ror #2	;-----X--
		orr r2,r2,r10,ror #4*2
		
		and r10,r9,r8,ror #3	;----X---
		orr r2,r2,r10,ror #4*3
		
		and r10,r9,r8,ror #4	;---X----
		orr r2,r2,r10,ror #4*4
		
		and r10,r9,r8,ror #5	;--X-----
		orr r2,r2,r10,ror #4*5
		
		and r10,r9,r8,ror #6	;-X------
		orr r2,r2,r10,ror #4*6
		
		and r10,r9,r8,ror #7	;X-------
		orr r2,r2,r10,ror #4*7
		
		mov r5,r2
		orr r5,r5,r2,asl #1		;Duplicate bit into all 4 bits of nibble
		orr r5,r5,r2,asl #2			;Color 1 -> Color 15
		orr r5,r5,r2,asl #3
		
		str r5,[r3]				;Store the complete font (in color 15)
		
		add r3,r3,#160;-8
		
		subs r1,r1,#1
		bne NextFontLine		;Repeat for next line
		
		mov r3,#ramarea
		ldrB r0,[r3,#CursorX]	;Inc X-pos
		add r0,r0,#1
		strB r0,[r3,#CursorX]	
	LDMFD sp!,{r0-r12, pc}		

NewLine:
	STMFD sp!,{r0-r12, lr}
		mov r3,#RamArea			;Pointer to Ram area
		
		mov r0,#0
		strB r0,[r3,#CursorX]	;Zero XPos
		
		ldrB r0,[r3,#CursorY]	;Inc Ypos
		add r0,r0,#1
		strB r0,[r3,#CursorY]	
	LDMFD sp!,{r0-r12, pc}		
	

Locate:	;X,Y = R1,R2
	STMFD sp!,{r0-r12, lr}
		mov r3,#RamArea			;Pointer to Ram area
		
		strB r1,[r3,#CursorX]	;Zero XPos
		strB r2,[r3,#CursorY]	;Inc Ypos
	LDMFD sp!,{r0-r12, pc}			
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	
ChibiSound:
	mov pc,lr	
	
	
	
ReadJoystick:
	STMFD sp!,{r1-r10, lr}
		mov r9,#255
	
		mov r10,#0b11111110
		mov r1,#255^57		;Key 57=UP (255 xor key)	
		bl Key_TestOne
		
		mov r10,#0b11111101
		mov r1,#255^41		;Key 41=DOWN (255 xor key)	
		bl Key_TestOne
		
		mov r10,#0b11111011
		mov r1,#255^25		;Key 25=LEFT (255 xor key)	
		bl Key_TestOne
		
		mov r10,#0b11110111
		mov r1,#255^121		;Key 25=RIGHT (255 xor key)	
		bl Key_TestOne
		
		
		mov r10,#0b11101111
		mov r1,#255^98		;Key 98=SPACE (255 xor key)	
		bl Key_TestOne
		
		mov r10,#0b11011111
		mov r1,#255^73		;Key 73=ENTER (255 xor key)	
		bl Key_TestOne
		
		mov r0,r9
	LDMFD sp!,{r1-r10, pc}		
	
Key_TestOne:
	STMFD sp!,{r0-r8, lr}
		mov r0,#129			;Test a key
		mov r2,#255			;255=No Wait
		SWI 0x6				;OSByte
		cmp r1,#255			;255=Pressed 0=Unpressed
		andeq r9,r9,r10 
	LDMFD sp!,{r0-r8, pc}		

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
BitmapFont:
        .byte 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00     ;  0
        .byte 0x10,0x18,0x18,0x18,0x18,0x00,0x18,0x00     ;  1
        .byte 0x28,0x6C,0x28,0x00,0x00,0x00,0x00,0x00     ;  2
        .byte 0x00,0x28,0x7C,0x28,0x7C,0x28,0x00,0x00     ;  3
        .byte 0x18,0x3E,0x48,0x3C,0x12,0x7C,0x18,0x00     ;  4
        .byte 0x02,0xC4,0xC8,0x10,0x20,0x46,0x86,0x00     ;  5
        .byte 0x10,0x28,0x28,0x72,0x94,0x8C,0x72,0x00     ;  6
        .byte 0x0C,0x1C,0x30,0x00,0x00,0x00,0x00,0x00     ;  7
        .byte 0x18,0x18,0x30,0x30,0x30,0x18,0x18,0x00     ;  8
        .byte 0x18,0x18,0x0C,0x0C,0x0C,0x18,0x18,0x00     ;  9
        .byte 0x08,0x49,0x2A,0x1C,0x14,0x22,0x41,0x00     ; 10
        .byte 0x00,0x18,0x18,0x7E,0x18,0x18,0x00,0x00     ; 11
        .byte 0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x30     ; 12
        .byte 0x00,0x00,0x00,0x7E,0x7E,0x00,0x00,0x00     ; 13
        .byte 0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x00     ; 14
        .byte 0x02,0x04,0x08,0x10,0x20,0x40,0x80,0x00     ; 15
        .byte 0x7C,0xC6,0xD6,0xD6,0xD6,0xC6,0x7C,0x00     ; 16
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 17
        .byte 0x3C,0x7E,0x06,0x3C,0x60,0x7E,0x3C,0x00     ; 18
        .byte 0x3C,0x7E,0x06,0x1C,0x06,0x7E,0x3C,0x00     ; 19
        .byte 0x18,0x3C,0x64,0xCC,0x7C,0x0C,0x08,0x00     ; 20
        .byte 0x3C,0x7E,0x60,0x7C,0x06,0x7E,0x3E,0x00     ; 21
        .byte 0x3C,0x7E,0x60,0x7C,0x66,0x66,0x3C,0x00     ; 22
        .byte 0x3C,0x7E,0x06,0x0C,0x18,0x18,0x10,0x00     ; 23
        .byte 0x3C,0x66,0x66,0x3C,0x66,0x66,0x3C,0x00     ; 24
        .byte 0x3C,0x66,0x66,0x3E,0x06,0x7E,0x3C,0x00     ; 25
        .byte 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x00     ; 26
        .byte 0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x30     ; 27
        .byte 0x0C,0x1C,0x38,0x60,0x38,0x1C,0x0C,0x00     ; 28
        .byte 0x00,0x00,0x7E,0x00,0x00,0x7E,0x00,0x00     ; 29
        .byte 0x60,0x70,0x38,0x0C,0x38,0x70,0x60,0x00     ; 30
        .byte 0x3C,0x76,0x06,0x1C,0x00,0x18,0x18,0x00     ; 31
        .byte 0x7C,0xCE,0xA6,0xB6,0xC6,0xF0,0x7C,0x00     ; 32
        .byte 0x18,0x3C,0x66,0x66,0x7E,0x66,0x24,0x00     ; 33
        .byte 0x3C,0x66,0x66,0x7C,0x66,0x66,0x3C,0x00     ; 34
        .byte 0x38,0x7C,0xC0,0xC0,0xC0,0x7C,0x38,0x00     ; 35
        .byte 0x3C,0x64,0x66,0x66,0x66,0x64,0x38,0x00     ; 36
        .byte 0x3C,0x7E,0x60,0x78,0x60,0x7E,0x3C,0x00     ; 37
        .byte 0x38,0x7C,0x60,0x78,0x60,0x60,0x20,0x00     ; 38
        .byte 0x3C,0x66,0xC0,0xC0,0xCC,0x66,0x3C,0x00     ; 39
        .byte 0x24,0x66,0x66,0x7E,0x66,0x66,0x24,0x00     ; 40
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 41
        .byte 0x08,0x0C,0x0C,0x0C,0x4C,0xFC,0x78,0x00     ; 42
        .byte 0x24,0x66,0x6C,0x78,0x6C,0x66,0x24,0x00     ; 43
        .byte 0x20,0x60,0x60,0x60,0x60,0x7E,0x3E,0x00     ; 44
        .byte 0x44,0xEE,0xFE,0xD6,0xD6,0xD6,0x44,0x00     ; 45
        .byte 0x44,0xE6,0xF6,0xDE,0xCE,0xC6,0x44,0x00     ; 46
        .byte 0x38,0x6C,0xC6,0xC6,0xC6,0x6C,0x38,0x00     ; 47
        .byte 0x38,0x6C,0x64,0x7C,0x60,0x60,0x20,0x00     ; 48
        .byte 0x38,0x6C,0xC6,0xC6,0xCA,0x74,0x3A,0x00     ; 49
        .byte 0x3C,0x66,0x66,0x7C,0x6C,0x66,0x26,0x00     ; 50
        .byte 0x3C,0x7E,0x60,0x3C,0x06,0x7E,0x3C,0x00     ; 51
        .byte 0x3C,0x7E,0x18,0x18,0x18,0x18,0x08,0x00     ; 52
        .byte 0x24,0x66,0x66,0x66,0x66,0x66,0x3C,0x00     ; 53
        .byte 0x24,0x66,0x66,0x66,0x66,0x3C,0x18,0x00     ; 54
        .byte 0x44,0xC6,0xD6,0xD6,0xFE,0xEE,0x44,0x00     ; 55
        .byte 0xC6,0x6C,0x38,0x38,0x6C,0xC6,0x44,0x00     ; 56
        .byte 0x24,0x66,0x66,0x3C,0x18,0x18,0x08,0x00     ; 57
        .byte 0x7C,0xFC,0x0C,0x18,0x30,0x7E,0x7C,0x00     ; 58
        .byte 0x1C,0x30,0x30,0x30,0x30,0x30,0x1C,0x00     ; 59
        .byte 0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x00     ; 60
        .byte 0x38,0x0C,0x0C,0x0C,0x0C,0x0C,0x38,0x00     ; 61
        .byte 0x18,0x18,0x18,0x18,0x7E,0x7E,0x18,0x18     ; 62
        .byte 0x18,0x18,0x18,0x18,0x3C,0x3C,0x18,0x18     ; 63
        .byte 0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18     ; 64
        .byte 0x00,0x00,0x38,0x0C,0x7C,0xCC,0x78,0x00     ; 65
        .byte 0x20,0x60,0x7C,0x66,0x66,0x66,0x3C,0x00     ; 66
        .byte 0x00,0x00,0x3C,0x66,0x60,0x66,0x3C,0x00     ; 67
        .byte 0x08,0x0C,0x7C,0xCC,0xCC,0xCC,0x78,0x00     ; 68
        .byte 0x00,0x00,0x3C,0x66,0x7E,0x60,0x3C,0x00     ; 69
        .byte 0x1C,0x36,0x30,0x38,0x30,0x30,0x10,0x00     ; 70
        .byte 0x00,0x00,0x3C,0x66,0x66,0x3E,0x06,0x3C     ; 71
        .byte 0x20,0x60,0x6C,0x76,0x66,0x66,0x24,0x00     ; 72
        .byte 0x18,0x00,0x18,0x18,0x18,0x18,0x08,0x00     ; 73
        .byte 0x06,0x00,0x04,0x06,0x06,0x26,0x66,0x3C     ; 74
        .byte 0x20,0x60,0x66,0x6C,0x78,0x6C,0x26,0x00     ; 75
        .byte 0x10,0x18,0x18,0x18,0x18,0x18,0x08,0x00     ; 76
        .byte 0x00,0x00,0x6C,0xFE,0xD6,0xD6,0xC6,0x00     ; 77
        .byte 0x00,0x00,0x3C,0x66,0x66,0x66,0x24,0x00     ; 78
        .byte 0x00,0x00,0x3C,0x66,0x66,0x66,0x3C,0x00     ; 79
        .byte 0x00,0x00,0x3C,0x66,0x66,0x7C,0x60,0x20     ; 80
        .byte 0x00,0x00,0x78,0xCC,0xCC,0x7C,0x0C,0x08     ; 81
        .byte 0x00,0x00,0x38,0x7C,0x60,0x60,0x20,0x00     ; 82
        .byte 0x00,0x00,0x3C,0x60,0x3C,0x06,0x7C,0x00     ; 83
        .byte 0x10,0x30,0x3C,0x30,0x30,0x3E,0x1C,0x00     ; 84
        .byte 0x00,0x00,0x24,0x66,0x66,0x66,0x3C,0x00     ; 85
        .byte 0x00,0x00,0x24,0x66,0x66,0x3C,0x18,0x00     ; 86
        .byte 0x00,0x00,0x44,0xD6,0xD6,0xFE,0x6C,0x00     ; 87
        .byte 0x00,0x00,0xC6,0x6C,0x38,0x6C,0xC6,0x00     ; 88
        .byte 0x00,0x00,0x24,0x66,0x66,0x3E,0x06,0x7C     ; 89
        .byte 0x00,0x00,0x7E,0x0C,0x18,0x30,0x7E,0x00     ; 90
        .byte 0x08,0x08,0x08,0x08,0x56,0x55,0x57,0x74     ; 91
        .byte 0x18,0x04,0x08,0x1C,0x56,0x55,0x57,0x74     ; 92
        .byte 0x00,0x00,0x00,0x00,0x7E,0x7E,0xFF,0xFF     ; 93
        .byte 0x18,0x3C,0x18,0x18,0x18,0x18,0x7E,0xFF     ; 94
        .byte 0x22,0x77,0x7F,0x7F,0x3E,0x1C,0x08,0x00     ; 95
 
			